# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/meta_graph.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from tensorboard.compat.proto import graph_pb2 as tensorboard_dot_compat_dot_proto_dot_graph__pb2
from tensorboard.compat.proto import op_def_pb2 as tensorboard_dot_compat_dot_proto_dot_op__def__pb2
from tensorboard.compat.proto import tensor_shape_pb2 as tensorboard_dot_compat_dot_proto_dot_tensor__shape__pb2
from tensorboard.compat.proto import types_pb2 as tensorboard_dot_compat_dot_proto_dot_types__pb2
from tensorboard.compat.proto import saved_object_graph_pb2 as tensorboard_dot_compat_dot_proto_dot_saved__object__graph__pb2
from tensorboard.compat.proto import saver_pb2 as tensorboard_dot_compat_dot_proto_dot_saver__pb2
from tensorboard.compat.proto import struct_pb2 as tensorboard_dot_compat_dot_proto_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)tensorboard/compat/proto/meta_graph.proto\x12\x0btensorboard\x1a\x19google/protobuf/any.proto\x1a$tensorboard/compat/proto/graph.proto\x1a%tensorboard/compat/proto/op_def.proto\x1a+tensorboard/compat/proto/tensor_shape.proto\x1a$tensorboard/compat/proto/types.proto\x1a\x31tensorboard/compat/proto/saved_object_graph.proto\x1a$tensorboard/compat/proto/saver.proto\x1a%tensorboard/compat/proto/struct.proto\"\xb3\x07\n\x0cMetaGraphDef\x12<\n\rmeta_info_def\x18\x01 \x01(\x0b\x32%.tensorboard.MetaGraphDef.MetaInfoDef\x12(\n\tgraph_def\x18\x02 \x01(\x0b\x32\x15.tensorboard.GraphDef\x12(\n\tsaver_def\x18\x03 \x01(\x0b\x32\x15.tensorboard.SaverDef\x12\x44\n\x0e\x63ollection_def\x18\x04 \x03(\x0b\x32,.tensorboard.MetaGraphDef.CollectionDefEntry\x12\x42\n\rsignature_def\x18\x05 \x03(\x0b\x32+.tensorboard.MetaGraphDef.SignatureDefEntry\x12\x31\n\x0e\x61sset_file_def\x18\x06 \x03(\x0b\x32\x19.tensorboard.AssetFileDef\x12\x37\n\x10object_graph_def\x18\x07 \x01(\x0b\x32\x1d.tensorboard.SavedObjectGraph\x1a\xf8\x02\n\x0bMetaInfoDef\x12\x1a\n\x12meta_graph_version\x18\x01 \x01(\t\x12-\n\x10stripped_op_list\x18\x02 \x01(\x0b\x32\x13.tensorboard.OpList\x12&\n\x08\x61ny_info\x18\x03 \x01(\x0b\x32\x14.google.protobuf.Any\x12\x0c\n\x04tags\x18\x04 \x03(\t\x12\x1a\n\x12tensorflow_version\x18\x05 \x01(\t\x12\x1e\n\x16tensorflow_git_version\x18\x06 \x01(\t\x12\x1e\n\x16stripped_default_attrs\x18\x07 \x01(\x08\x12T\n\x10\x66unction_aliases\x18\x08 \x03(\x0b\x32:.tensorboard.MetaGraphDef.MetaInfoDef.FunctionAliasesEntry\x1a\x36\n\x14\x46unctionAliasesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1aP\n\x12\x43ollectionDefEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.tensorboard.CollectionDef:\x02\x38\x01\x1aN\n\x11SignatureDefEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.tensorboard.SignatureDef:\x02\x38\x01\"\xe4\x03\n\rCollectionDef\x12\x38\n\tnode_list\x18\x01 \x01(\x0b\x32#.tensorboard.CollectionDef.NodeListH\x00\x12:\n\nbytes_list\x18\x02 \x01(\x0b\x32$.tensorboard.CollectionDef.BytesListH\x00\x12:\n\nint64_list\x18\x03 \x01(\x0b\x32$.tensorboard.CollectionDef.Int64ListH\x00\x12:\n\nfloat_list\x18\x04 \x01(\x0b\x32$.tensorboard.CollectionDef.FloatListH\x00\x12\x36\n\x08\x61ny_list\x18\x05 \x01(\x0b\x32\".tensorboard.CollectionDef.AnyListH\x00\x1a\x19\n\x08NodeList\x12\r\n\x05value\x18\x01 \x03(\t\x1a\x1a\n\tBytesList\x12\r\n\x05value\x18\x01 \x03(\x0c\x1a\x1e\n\tInt64List\x12\x11\n\x05value\x18\x01 \x03(\x03\x42\x02\x10\x01\x1a\x1e\n\tFloatList\x12\x11\n\x05value\x18\x01 \x03(\x02\x42\x02\x10\x01\x1a.\n\x07\x41nyList\x12#\n\x05value\x18\x01 \x03(\x0b\x32\x14.google.protobuf.AnyB\x06\n\x04kind\"\xd7\x03\n\nTensorInfo\x12\x0e\n\x04name\x18\x01 \x01(\tH\x00\x12\x37\n\ncoo_sparse\x18\x04 \x01(\x0b\x32!.tensorboard.TensorInfo.CooSparseH\x00\x12\x43\n\x10\x63omposite_tensor\x18\x05 \x01(\x0b\x32\'.tensorboard.TensorInfo.CompositeTensorH\x00\x12$\n\x05\x64type\x18\x02 \x01(\x0e\x32\x15.tensorboard.DataType\x12\x33\n\x0ctensor_shape\x18\x03 \x01(\x0b\x32\x1d.tensorboard.TensorShapeProto\x1a\x65\n\tCooSparse\x12\x1a\n\x12values_tensor_name\x18\x01 \x01(\t\x12\x1b\n\x13indices_tensor_name\x18\x02 \x01(\t\x12\x1f\n\x17\x64\x65nse_shape_tensor_name\x18\x03 \x01(\t\x1am\n\x0f\x43ompositeTensor\x12-\n\ttype_spec\x18\x01 \x01(\x0b\x32\x1a.tensorboard.TypeSpecProto\x12+\n\ncomponents\x18\x02 \x03(\x0b\x32\x17.tensorboard.TensorInfoB\n\n\x08\x65ncoding\"\xa4\x02\n\x0cSignatureDef\x12\x35\n\x06inputs\x18\x01 \x03(\x0b\x32%.tensorboard.SignatureDef.InputsEntry\x12\x37\n\x07outputs\x18\x02 \x03(\x0b\x32&.tensorboard.SignatureDef.OutputsEntry\x12\x13\n\x0bmethod_name\x18\x03 \x01(\t\x1a\x46\n\x0bInputsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.tensorboard.TensorInfo:\x02\x38\x01\x1aG\n\x0cOutputsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.tensorboard.TensorInfo:\x02\x38\x01\"N\n\x0c\x41ssetFileDef\x12,\n\x0btensor_info\x18\x01 \x01(\x0b\x32\x17.tensorboard.TensorInfo\x12\x10\n\x08\x66ilename\x18\x02 \x01(\tB\x87\x01\n\x18org.tensorflow.frameworkB\x0fMetaGraphProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01\x62\x06proto3')



_METAGRAPHDEF = DESCRIPTOR.message_types_by_name['MetaGraphDef']
_METAGRAPHDEF_METAINFODEF = _METAGRAPHDEF.nested_types_by_name['MetaInfoDef']
_METAGRAPHDEF_METAINFODEF_FUNCTIONALIASESENTRY = _METAGRAPHDEF_METAINFODEF.nested_types_by_name['FunctionAliasesEntry']
_METAGRAPHDEF_COLLECTIONDEFENTRY = _METAGRAPHDEF.nested_types_by_name['CollectionDefEntry']
_METAGRAPHDEF_SIGNATUREDEFENTRY = _METAGRAPHDEF.nested_types_by_name['SignatureDefEntry']
_COLLECTIONDEF = DESCRIPTOR.message_types_by_name['CollectionDef']
_COLLECTIONDEF_NODELIST = _COLLECTIONDEF.nested_types_by_name['NodeList']
_COLLECTIONDEF_BYTESLIST = _COLLECTIONDEF.nested_types_by_name['BytesList']
_COLLECTIONDEF_INT64LIST = _COLLECTIONDEF.nested_types_by_name['Int64List']
_COLLECTIONDEF_FLOATLIST = _COLLECTIONDEF.nested_types_by_name['FloatList']
_COLLECTIONDEF_ANYLIST = _COLLECTIONDEF.nested_types_by_name['AnyList']
_TENSORINFO = DESCRIPTOR.message_types_by_name['TensorInfo']
_TENSORINFO_COOSPARSE = _TENSORINFO.nested_types_by_name['CooSparse']
_TENSORINFO_COMPOSITETENSOR = _TENSORINFO.nested_types_by_name['CompositeTensor']
_SIGNATUREDEF = DESCRIPTOR.message_types_by_name['SignatureDef']
_SIGNATUREDEF_INPUTSENTRY = _SIGNATUREDEF.nested_types_by_name['InputsEntry']
_SIGNATUREDEF_OUTPUTSENTRY = _SIGNATUREDEF.nested_types_by_name['OutputsEntry']
_ASSETFILEDEF = DESCRIPTOR.message_types_by_name['AssetFileDef']
MetaGraphDef = _reflection.GeneratedProtocolMessageType('MetaGraphDef', (_message.Message,), {

  'MetaInfoDef' : _reflection.GeneratedProtocolMessageType('MetaInfoDef', (_message.Message,), {

    'FunctionAliasesEntry' : _reflection.GeneratedProtocolMessageType('FunctionAliasesEntry', (_message.Message,), {
      'DESCRIPTOR' : _METAGRAPHDEF_METAINFODEF_FUNCTIONALIASESENTRY,
      '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
      # @@protoc_insertion_point(class_scope:tensorboard.MetaGraphDef.MetaInfoDef.FunctionAliasesEntry)
      })
    ,
    'DESCRIPTOR' : _METAGRAPHDEF_METAINFODEF,
    '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.MetaGraphDef.MetaInfoDef)
    })
  ,

  'CollectionDefEntry' : _reflection.GeneratedProtocolMessageType('CollectionDefEntry', (_message.Message,), {
    'DESCRIPTOR' : _METAGRAPHDEF_COLLECTIONDEFENTRY,
    '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.MetaGraphDef.CollectionDefEntry)
    })
  ,

  'SignatureDefEntry' : _reflection.GeneratedProtocolMessageType('SignatureDefEntry', (_message.Message,), {
    'DESCRIPTOR' : _METAGRAPHDEF_SIGNATUREDEFENTRY,
    '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.MetaGraphDef.SignatureDefEntry)
    })
  ,
  'DESCRIPTOR' : _METAGRAPHDEF,
  '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.MetaGraphDef)
  })
_sym_db.RegisterMessage(MetaGraphDef)
_sym_db.RegisterMessage(MetaGraphDef.MetaInfoDef)
_sym_db.RegisterMessage(MetaGraphDef.MetaInfoDef.FunctionAliasesEntry)
_sym_db.RegisterMessage(MetaGraphDef.CollectionDefEntry)
_sym_db.RegisterMessage(MetaGraphDef.SignatureDefEntry)

CollectionDef = _reflection.GeneratedProtocolMessageType('CollectionDef', (_message.Message,), {

  'NodeList' : _reflection.GeneratedProtocolMessageType('NodeList', (_message.Message,), {
    'DESCRIPTOR' : _COLLECTIONDEF_NODELIST,
    '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.CollectionDef.NodeList)
    })
  ,

  'BytesList' : _reflection.GeneratedProtocolMessageType('BytesList', (_message.Message,), {
    'DESCRIPTOR' : _COLLECTIONDEF_BYTESLIST,
    '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.CollectionDef.BytesList)
    })
  ,

  'Int64List' : _reflection.GeneratedProtocolMessageType('Int64List', (_message.Message,), {
    'DESCRIPTOR' : _COLLECTIONDEF_INT64LIST,
    '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.CollectionDef.Int64List)
    })
  ,

  'FloatList' : _reflection.GeneratedProtocolMessageType('FloatList', (_message.Message,), {
    'DESCRIPTOR' : _COLLECTIONDEF_FLOATLIST,
    '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.CollectionDef.FloatList)
    })
  ,

  'AnyList' : _reflection.GeneratedProtocolMessageType('AnyList', (_message.Message,), {
    'DESCRIPTOR' : _COLLECTIONDEF_ANYLIST,
    '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.CollectionDef.AnyList)
    })
  ,
  'DESCRIPTOR' : _COLLECTIONDEF,
  '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.CollectionDef)
  })
_sym_db.RegisterMessage(CollectionDef)
_sym_db.RegisterMessage(CollectionDef.NodeList)
_sym_db.RegisterMessage(CollectionDef.BytesList)
_sym_db.RegisterMessage(CollectionDef.Int64List)
_sym_db.RegisterMessage(CollectionDef.FloatList)
_sym_db.RegisterMessage(CollectionDef.AnyList)

TensorInfo = _reflection.GeneratedProtocolMessageType('TensorInfo', (_message.Message,), {

  'CooSparse' : _reflection.GeneratedProtocolMessageType('CooSparse', (_message.Message,), {
    'DESCRIPTOR' : _TENSORINFO_COOSPARSE,
    '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.TensorInfo.CooSparse)
    })
  ,

  'CompositeTensor' : _reflection.GeneratedProtocolMessageType('CompositeTensor', (_message.Message,), {
    'DESCRIPTOR' : _TENSORINFO_COMPOSITETENSOR,
    '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.TensorInfo.CompositeTensor)
    })
  ,
  'DESCRIPTOR' : _TENSORINFO,
  '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.TensorInfo)
  })
_sym_db.RegisterMessage(TensorInfo)
_sym_db.RegisterMessage(TensorInfo.CooSparse)
_sym_db.RegisterMessage(TensorInfo.CompositeTensor)

SignatureDef = _reflection.GeneratedProtocolMessageType('SignatureDef', (_message.Message,), {

  'InputsEntry' : _reflection.GeneratedProtocolMessageType('InputsEntry', (_message.Message,), {
    'DESCRIPTOR' : _SIGNATUREDEF_INPUTSENTRY,
    '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.SignatureDef.InputsEntry)
    })
  ,

  'OutputsEntry' : _reflection.GeneratedProtocolMessageType('OutputsEntry', (_message.Message,), {
    'DESCRIPTOR' : _SIGNATUREDEF_OUTPUTSENTRY,
    '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.SignatureDef.OutputsEntry)
    })
  ,
  'DESCRIPTOR' : _SIGNATUREDEF,
  '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.SignatureDef)
  })
_sym_db.RegisterMessage(SignatureDef)
_sym_db.RegisterMessage(SignatureDef.InputsEntry)
_sym_db.RegisterMessage(SignatureDef.OutputsEntry)

AssetFileDef = _reflection.GeneratedProtocolMessageType('AssetFileDef', (_message.Message,), {
  'DESCRIPTOR' : _ASSETFILEDEF,
  '__module__' : 'tensorboard.compat.proto.meta_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.AssetFileDef)
  })
_sym_db.RegisterMessage(AssetFileDef)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030org.tensorflow.frameworkB\017MetaGraphProtosP\001ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\370\001\001'
  _METAGRAPHDEF_METAINFODEF_FUNCTIONALIASESENTRY._options = None
  _METAGRAPHDEF_METAINFODEF_FUNCTIONALIASESENTRY._serialized_options = b'8\001'
  _METAGRAPHDEF_COLLECTIONDEFENTRY._options = None
  _METAGRAPHDEF_COLLECTIONDEFENTRY._serialized_options = b'8\001'
  _METAGRAPHDEF_SIGNATUREDEFENTRY._options = None
  _METAGRAPHDEF_SIGNATUREDEFENTRY._serialized_options = b'8\001'
  _COLLECTIONDEF_INT64LIST.fields_by_name['value']._options = None
  _COLLECTIONDEF_INT64LIST.fields_by_name['value']._serialized_options = b'\020\001'
  _COLLECTIONDEF_FLOATLIST.fields_by_name['value']._options = None
  _COLLECTIONDEF_FLOATLIST.fields_by_name['value']._serialized_options = b'\020\001'
  _SIGNATUREDEF_INPUTSENTRY._options = None
  _SIGNATUREDEF_INPUTSENTRY._serialized_options = b'8\001'
  _SIGNATUREDEF_OUTPUTSENTRY._options = None
  _SIGNATUREDEF_OUTPUTSENTRY._serialized_options = b'8\001'
  _METAGRAPHDEF._serialized_start=374
  _METAGRAPHDEF._serialized_end=1321
  _METAGRAPHDEF_METAINFODEF._serialized_start=783
  _METAGRAPHDEF_METAINFODEF._serialized_end=1159
  _METAGRAPHDEF_METAINFODEF_FUNCTIONALIASESENTRY._serialized_start=1105
  _METAGRAPHDEF_METAINFODEF_FUNCTIONALIASESENTRY._serialized_end=1159
  _METAGRAPHDEF_COLLECTIONDEFENTRY._serialized_start=1161
  _METAGRAPHDEF_COLLECTIONDEFENTRY._serialized_end=1241
  _METAGRAPHDEF_SIGNATUREDEFENTRY._serialized_start=1243
  _METAGRAPHDEF_SIGNATUREDEFENTRY._serialized_end=1321
  _COLLECTIONDEF._serialized_start=1324
  _COLLECTIONDEF._serialized_end=1808
  _COLLECTIONDEF_NODELIST._serialized_start=1635
  _COLLECTIONDEF_NODELIST._serialized_end=1660
  _COLLECTIONDEF_BYTESLIST._serialized_start=1662
  _COLLECTIONDEF_BYTESLIST._serialized_end=1688
  _COLLECTIONDEF_INT64LIST._serialized_start=1690
  _COLLECTIONDEF_INT64LIST._serialized_end=1720
  _COLLECTIONDEF_FLOATLIST._serialized_start=1722
  _COLLECTIONDEF_FLOATLIST._serialized_end=1752
  _COLLECTIONDEF_ANYLIST._serialized_start=1754
  _COLLECTIONDEF_ANYLIST._serialized_end=1800
  _TENSORINFO._serialized_start=1811
  _TENSORINFO._serialized_end=2282
  _TENSORINFO_COOSPARSE._serialized_start=2058
  _TENSORINFO_COOSPARSE._serialized_end=2159
  _TENSORINFO_COMPOSITETENSOR._serialized_start=2161
  _TENSORINFO_COMPOSITETENSOR._serialized_end=2270
  _SIGNATUREDEF._serialized_start=2285
  _SIGNATUREDEF._serialized_end=2577
  _SIGNATUREDEF_INPUTSENTRY._serialized_start=2434
  _SIGNATUREDEF_INPUTSENTRY._serialized_end=2504
  _SIGNATUREDEF_OUTPUTSENTRY._serialized_start=2506
  _SIGNATUREDEF_OUTPUTSENTRY._serialized_end=2577
  _ASSETFILEDEF._serialized_start=2579
  _ASSETFILEDEF._serialized_end=2657
# @@protoc_insertion_point(module_scope)



# Copyright 2020 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Helpers that format the information about experiments as strings."""


import abc
import collections
import json

from tensorboard.uploader import util


class BaseExperimentFormatter:
    """Abstract base class for formatting experiment information as a string."""

    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def format_experiment(self, experiment, experiment_url):
        """Format the information about an experiment as a representing string.

        Args:
          experiment: An `experiment_pb2.Experiment` protobuf message for the
            experiment to be formatted.
          experiment_url: The URL at which the experiment can be accessed via
            TensorBoard.

        Returns:
          A string that represents the experiment.
        """
        pass


class ReadableFormatter(BaseExperimentFormatter):
    """A formatter implementation that outputs human-readable text."""

    _NAME_COLUMN_WIDTH = 20

    def __init__(self):
        super().__init__()

    def format_experiment(self, experiment, experiment_url):
        output = []
        output.append(experiment_url)
        data = [
            ("Name", experiment.name or "[No Name]"),
            ("Description", experiment.description or "[No Description]"),
            ("Id", experiment.experiment_id),
            ("Created", util.format_time(experiment.create_time)),
            ("Updated", util.format_time(experiment.update_time)),
            ("Runs", str(experiment.num_runs)),
            ("Tags", str(experiment.num_tags)),
            ("Scalars", str(experiment.num_scalars)),
            ("Tensor bytes", str(experiment.total_tensor_bytes)),
            ("Binary object bytes", str(experiment.total_blob_bytes)),
        ]
        for name, value in data:
            output.append(
                "\t%s %s"
                % (
                    name.ljust(self._NAME_COLUMN_WIDTH),
                    value,
                )
            )
        return "\n".join(output)


class JsonFormatter:
    """A formatter implementation: outputs experiment as JSON."""

    _JSON_INDENT = 2

    def __init__(self):
        super().__init__()

    def format_experiment(self, experiment, experiment_url):
        data = [
            ("url", experiment_url),
            ("name", experiment.name),
            ("description", experiment.description),
            ("id", experiment.experiment_id),
            ("created", util.format_time_absolute(experiment.create_time)),
            ("updated", util.format_time_absolute(experiment.update_time)),
            ("runs", experiment.num_runs),
            ("tags", experiment.num_tags),
            ("scalars", experiment.num_scalars),
            ("tensor_bytes", experiment.total_tensor_bytes),
            ("binary_object_bytes", experiment.total_blob_bytes),
        ]
        return json.dumps(
            collections.OrderedDict(data),
            indent=self._JSON_INDENT,
        )



# Copyright 2019 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Utilities for graph plugin."""

from tensorboard.compat.proto import graph_pb2


def _prefixed_op_name(prefix, op_name):
    return "%s/%s" % (prefix, op_name)


def _prefixed_func_name(prefix, func_name):
    """Returns function name prefixed with `prefix`.

    For function libraries, which are often created out of autographed Python
    function, are factored out in the graph vis. They are grouped under a
    function name which often has a shape of
    `__inference_[py_func_name]_[numeric_suffix]`.

    While it does not have some unique information about which graph it is from,
    creating another wrapping structure with graph prefix and "/" is less than
    ideal so we join the prefix and func_name using underscore.

    TODO(stephanwlee): add business logic to strip "__inference_" for more user
    friendlier name
    """
    return "%s_%s" % (prefix, func_name)


def _add_with_prepended_names(prefix, graph_to_add, destination_graph):
    for node in graph_to_add.node:
        new_node = destination_graph.node.add()
        new_node.CopyFrom(node)
        new_node.name = _prefixed_op_name(prefix, node.name)
        new_node.input[:] = [
            _prefixed_op_name(prefix, input_name) for input_name in node.input
        ]

        # Remap tf.function method name in the PartitionedCall. 'f' is short for
        # function.
        if new_node.op == "PartitionedCall" and new_node.attr["f"]:

            new_node.attr["f"].func.name = _prefixed_func_name(
                prefix,
                new_node.attr["f"].func.name,
            )

    for func in graph_to_add.library.function:
        new_func = destination_graph.library.function.add()
        new_func.CopyFrom(func)
        new_func.signature.name = _prefixed_func_name(
            prefix, new_func.signature.name
        )

    for gradient in graph_to_add.library.gradient:
        new_gradient = destination_graph.library.gradient.add()
        new_gradient.CopyFrom(gradient)
        new_gradient.function_name = _prefixed_func_name(
            prefix,
            new_gradient.function_name,
        )
        new_gradient.gradient_func = _prefixed_func_name(
            prefix,
            new_gradient.gradient_func,
        )


def merge_graph_defs(graph_defs):
    """Merges GraphDefs by adding unique prefix, `graph_{ind}`, to names.

    All GraphDefs are expected to be of TensorBoard's.

    When collecting graphs using the `tf.summary.trace` API, node names are not
    guranteed to be unique.  When non-unique names are not considered, it can
    lead to graph visualization showing them as one which creates inaccurate
    depiction of the flow of the graph (e.g., if there are A -> B -> C and D ->
    B -> E, you may see {A, D} -> B -> E).  To prevent such graph, we checked
    for uniquenss while merging but it resulted in
    https://github.com/tensorflow/tensorboard/issues/1929.

    To remedy these issues, we simply "apply name scope" on each graph by
    prefixing it with unique name (with a chance of collision) to create
    unconnected group of graphs.

    In case there is only one graph def passed, it returns the original
    graph_def. In case no graph defs are passed, it returns an empty GraphDef.

    Args:
      graph_defs: TensorBoard GraphDefs to merge.

    Returns:
      TensorBoard GraphDef that merges all graph_defs with unique prefixes.

    Raises:
      ValueError in case GraphDef versions mismatch.
    """
    if len(graph_defs) == 1:
        return graph_defs[0]
    elif len(graph_defs) == 0:
        return graph_pb2.GraphDef()

    dst_graph_def = graph_pb2.GraphDef()

    if graph_defs[0].versions.producer:
        dst_graph_def.versions.CopyFrom(graph_defs[0].versions)

    for index, graph_def in enumerate(graph_defs):
        if dst_graph_def.versions.producer != graph_def.versions.producer:
            raise ValueError("Cannot combine GraphDefs of different versions.")

        _add_with_prepended_names(
            "graph_%d" % (index + 1),
            graph_def,
            dst_graph_def,
        )

    return dst_graph_def



# Copyright 2018 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Histogram summaries and TensorFlow operations to create them, V2 versions.

A histogram summary stores a list of buckets. Each bucket is encoded as a triple
`[left_edge, right_edge, count]`. Thus, a full histogram is encoded as a tensor
of dimension `[k, 3]`, where the first `k - 1` buckets are closed-open and the
last bucket is closed-closed.

In general, the shape of the output histogram is always constant (`[k, 3]`).
In the case of empty data, the output will be an all-zero histogram of shape
`[k, 3]`, where all edges and counts are zeros. If there is data but all points
have the same value, then all buckets' left and right edges are the same and only
the last bucket has nonzero count.
"""

import numpy as np

from tensorboard.compat import tf2 as tf
from tensorboard.compat.proto import summary_pb2
from tensorboard.plugins.histogram import metadata
from tensorboard.util import lazy_tensor_creator
from tensorboard.util import tensor_util


DEFAULT_BUCKET_COUNT = 30


def histogram_pb(tag, data, buckets=None, description=None):
    """Create a histogram summary protobuf.

    Arguments:
      tag: String tag for the summary.
      data: A `np.array` or array-like form of any shape. Must have type
        castable to `float`.
      buckets: Optional positive `int`. The output shape will always be
        [buckets, 3]. If there is no data, then an all-zero array of shape
        [buckets, 3] will be returned. If there is data but all points have
        the same value, then all buckets' left and right endpoints are the
        same and only the last bucket has nonzero count. Defaults to 30 if
        not specified.
      description: Optional long-form description for this summary, as a
        `str`. Markdown is supported. Defaults to empty.

    Returns:
      A `summary_pb2.Summary` protobuf object.
    """
    bucket_count = DEFAULT_BUCKET_COUNT if buckets is None else buckets
    data = np.array(data).flatten().astype(float)
    if bucket_count == 0 or data.size == 0:
        histogram_buckets = np.zeros((bucket_count, 3))
    else:
        min_ = np.min(data)
        max_ = np.max(data)
        range_ = max_ - min_
        if range_ == 0:
            left_edges = right_edges = np.array([min_] * bucket_count)
            bucket_counts = np.array([0] * (bucket_count - 1) + [data.size])
            histogram_buckets = np.array(
                [left_edges, right_edges, bucket_counts]
            ).transpose()
        else:
            bucket_width = range_ / bucket_count
            offsets = data - min_
            bucket_indices = np.floor(offsets / bucket_width).astype(int)
            clamped_indices = np.minimum(bucket_indices, bucket_count - 1)
            one_hots = np.array([clamped_indices]).transpose() == np.arange(
                0, bucket_count
            )  # broadcast
            assert one_hots.shape == (data.size, bucket_count), (
                one_hots.shape,
                (data.size, bucket_count),
            )
            bucket_counts = np.sum(one_hots, axis=0)
            edges = np.linspace(min_, max_, bucket_count + 1)
            left_edges = edges[:-1]
            right_edges = edges[1:]
            histogram_buckets = np.array(
                [left_edges, right_edges, bucket_counts]
            ).transpose()
    tensor = tensor_util.make_tensor_proto(histogram_buckets, dtype=np.float64)

    summary_metadata = metadata.create_summary_metadata(
        display_name=None, description=description
    )
    summary = summary_pb2.Summary()
    summary.value.add(tag=tag, metadata=summary_metadata, tensor=tensor)
    return summary


# This is the TPU compatible V3 histogram implementation as of 2021-12-01.
def histogram(name, data, step=None, buckets=None, description=None):
    """Write a histogram summary.

    See also `tf.summary.scalar`, `tf.summary.SummaryWriter`.

    Writes a histogram to the current default summary writer, for later analysis
    in TensorBoard's 'Histograms' and 'Distributions' dashboards (data written
    using this API will appear in both places). Like `tf.summary.scalar` points,
    each histogram is associated with a `step` and a `name`. All the histograms
    with the same `name` constitute a time series of histograms.

    The histogram is calculated over all the elements of the given `Tensor`
    without regard to its shape or rank.

    This example writes 2 histograms:

    ```python
    w = tf.summary.create_file_writer('test/logs')
    with w.as_default():
        tf.summary.histogram("activations", tf.random.uniform([100, 50]), step=0)
        tf.summary.histogram("initial_weights", tf.random.normal([1000]), step=0)
    ```

    A common use case is to examine the changing activation patterns (or lack
    thereof) at specific layers in a neural network, over time.

    ```python
    w = tf.summary.create_file_writer('test/logs')
    with w.as_default():
    for step in range(100):
        # Generate fake "activations".
        activations = [
            tf.random.normal([1000], mean=step, stddev=1),
            tf.random.normal([1000], mean=step, stddev=10),
            tf.random.normal([1000], mean=step, stddev=100),
        ]

        tf.summary.histogram("layer1/activate", activations[0], step=step)
        tf.summary.histogram("layer2/activate", activations[1], step=step)
        tf.summary.histogram("layer3/activate", activations[2], step=step)
    ```

    Arguments:
      name: A name for this summary. The summary tag used for TensorBoard will
        be this name prefixed by any active name scopes.
      data: A `Tensor` of any shape. The histogram is computed over its elements,
        which must be castable to `float64`.
      step: Explicit `int64`-castable monotonic step value for this summary. If
        omitted, this defaults to `tf.summary.experimental.get_step()`, which must
        not be None.
      buckets: Optional positive `int`. The output will have this
        many buckets, except in two edge cases. If there is no data, then
        there are no buckets. If there is data but all points have the
        same value, then all buckets' left and right endpoints are the same
        and only the last bucket has nonzero count. Defaults to 30 if not
        specified.
      description: Optional long-form description for this summary, as a
        constant `str`. Markdown is supported. Defaults to empty.

    Returns:
      True on success, or false if no summary was emitted because no default
      summary writer was available.

    Raises:
      ValueError: if a default writer exists, but no step was provided and
        `tf.summary.experimental.get_step()` is None.
    """
    # Avoid building unused gradient graphs for conds below. This works around
    # an error building second-order gradient graphs when XlaDynamicUpdateSlice
    # is used, and will generally speed up graph building slightly.
    data = tf.stop_gradient(data)
    summary_metadata = metadata.create_summary_metadata(
        display_name=None, description=description
    )
    # TODO(https://github.com/tensorflow/tensorboard/issues/2109): remove fallback
    summary_scope = (
        getattr(tf.summary.experimental, "summary_scope", None)
        or tf.summary.summary_scope
    )

    # TODO(ytjing): add special case handling.
    with summary_scope(
        name, "histogram_summary", values=[data, buckets, step]
    ) as (tag, _):
        # Defer histogram bucketing logic by passing it as a callable to
        # write(), wrapped in a LazyTensorCreator for backwards
        # compatibility, so that we only do this work when summaries are
        # actually written.
        @lazy_tensor_creator.LazyTensorCreator
        def lazy_tensor():
            return _buckets(data, buckets)

        return tf.summary.write(
            tag=tag,
            tensor=lazy_tensor,
            step=step,
            metadata=summary_metadata,
        )


def _buckets(data, bucket_count=None):
    """Create a TensorFlow op to group data into histogram buckets.

    Arguments:
      data: A `Tensor` of any shape. Must be castable to `float64`.
      bucket_count: Optional non-negative `int` or scalar `int32` `Tensor`,
        defaults to 30.
    Returns:
      A `Tensor` of shape `[k, 3]` and type `float64`. The `i`th row is
      a triple `[left_edge, right_edge, count]` for a single bucket.
      The value of `k` is either `bucket_count` or `0` (when input data
      is empty).
    """
    if bucket_count is None:
        bucket_count = DEFAULT_BUCKET_COUNT
    with tf.name_scope("buckets"):
        tf.debugging.assert_scalar(bucket_count)
        tf.debugging.assert_type(bucket_count, tf.int32)
        # Treat a negative bucket count as zero.
        bucket_count = tf.math.maximum(0, bucket_count)
        data = tf.reshape(data, shape=[-1])  # flatten
        data = tf.cast(data, tf.float64)
        data_size = tf.size(input=data)
        is_empty = tf.logical_or(
            tf.equal(data_size, 0), tf.less_equal(bucket_count, 0)
        )

        def when_empty():
            """When input data is empty or bucket_count is zero.

            1. If bucket_count is specified as zero, an empty tensor of shape
              (0, 3) will be returned.
            2. If the input data is empty, a tensor of shape (bucket_count, 3)
              of all zero values will be returned.
            """
            return tf.zeros((bucket_count, 3), dtype=tf.float64)

        def when_nonempty():
            min_ = tf.reduce_min(input_tensor=data)
            max_ = tf.reduce_max(input_tensor=data)
            range_ = max_ - min_
            has_single_value = tf.equal(range_, 0)

            def when_multiple_values():
                """When input data contains multiple values."""
                bucket_width = range_ / tf.cast(bucket_count, tf.float64)
                offsets = data - min_
                bucket_indices = tf.cast(
                    tf.floor(offsets / bucket_width), dtype=tf.int32
                )
                clamped_indices = tf.minimum(bucket_indices, bucket_count - 1)
                # Use float64 instead of float32 to avoid accumulating floating point error
                # later in tf.reduce_sum when summing more than 2^24 individual `1.0` values.
                # See https://github.com/tensorflow/tensorflow/issues/51419 for details.
                one_hots = tf.one_hot(
                    clamped_indices, depth=bucket_count, dtype=tf.float64
                )
                bucket_counts = tf.cast(
                    tf.reduce_sum(input_tensor=one_hots, axis=0),
                    dtype=tf.float64,
                )
                edges = tf.linspace(min_, max_, bucket_count + 1)
                # Ensure edges[-1] == max_, which TF's linspace implementation does not
                # do, leaving it subject to the whim of floating point rounding error.
                edges = tf.concat([edges[:-1], [max_]], 0)
                left_edges = edges[:-1]
                right_edges = edges[1:]
                return tf.transpose(
                    a=tf.stack([left_edges, right_edges, bucket_counts])
                )

            def when_single_value():
                """When input data contains a single unique value."""
                # Left and right edges are the same for single value input.
                edges = tf.fill([bucket_count], max_)
                # Bucket counts are 0 except the last bucket (if bucket_count > 0),
                # which is `data_size`. Ensure that the resulting counts vector has
                # length `bucket_count` always, including the bucket_count==0 case.
                zeroes = tf.fill([bucket_count], 0)
                bucket_counts = tf.cast(
                    tf.concat([zeroes[:-1], [data_size]], 0)[:bucket_count],
                    dtype=tf.float64,
                )
                return tf.transpose(a=tf.stack([edges, edges, bucket_counts]))

            return tf.cond(
                has_single_value, when_single_value, when_multiple_values
            )

        return tf.cond(is_empty, when_empty, when_nonempty)



# Copyright 2015 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Exception types for TensorFlow errors."""


import traceback
import warnings

from . import error_codes


# @tf_export("OpError", "errors.OpError")
class OpError(Exception):
    """A generic error that is raised when TensorFlow execution fails.

    Whenever possible, the session will raise a more specific subclass
    of `OpError` from the `tf.errors` module.
    """

    def __init__(self, node_def, op, message, error_code):
        """Creates a new `OpError` indicating that a particular op failed.

        Args:
          node_def: The `node_def_pb2.NodeDef` proto representing the op that
            failed, if known; otherwise None.
          op: The `ops.Operation` that failed, if known; otherwise None.
          message: The message string describing the failure.
          error_code: The `error_codes.Code` describing the error.
        """
        super().__init__()
        self._message = message
        self._node_def = node_def
        self._op = op
        self._error_code = error_code

    @property
    def message(self):
        """The error message that describes the error."""
        return self._message

    @property
    def op(self):
        """The operation that failed, if known.

        *N.B.* If the failed op was synthesized at runtime, e.g. a `Send`
        or `Recv` op, there will be no corresponding
        @{tf.Operation}
        object.  In that case, this will return `None`, and you should
        instead use the @{tf.OpError.node_def} to
        discover information about the op.

        Returns:
          The `Operation` that failed, or None.
        """
        return self._op

    @property
    def error_code(self):
        """The integer error code that describes the error."""
        return self._error_code

    @property
    def node_def(self):
        """The `NodeDef` proto representing the op that failed."""
        return self._node_def

    def __str__(self):
        if self._op is not None:
            output = [
                "%s\n\nCaused by op %r, defined at:\n"
                % (self.message, self._op.name)
            ]
            curr_traceback_list = traceback.format_list(self._op.traceback)
            output.extend(curr_traceback_list)
            # pylint: disable=protected-access
            original_op = self._op._original_op
            # pylint: enable=protected-access
            while original_op is not None:
                output.append(
                    "\n...which was originally created as op %r, defined at:\n"
                    % (original_op.name,)
                )
                prev_traceback_list = curr_traceback_list
                curr_traceback_list = traceback.format_list(
                    original_op.traceback
                )

                # Attempt to elide large common subsequences of the subsequent
                # stack traces.
                #
                # TODO(mrry): Consider computing the actual longest common subsequence.
                is_eliding = False
                elide_count = 0
                last_elided_line = None
                for line, line_in_prev in zip(
                    curr_traceback_list, prev_traceback_list
                ):
                    if line == line_in_prev:
                        if is_eliding:
                            elide_count += 1
                            last_elided_line = line
                        else:
                            output.append(line)
                            is_eliding = True
                            elide_count = 0
                    else:
                        if is_eliding:
                            if elide_count > 0:
                                output.extend(
                                    [
                                        "[elided %d identical lines from previous traceback]\n"
                                        % (elide_count - 1,),
                                        last_elided_line,
                                    ]
                                )
                            is_eliding = False
                        output.extend(line)

                # pylint: disable=protected-access
                original_op = original_op._original_op
                # pylint: enable=protected-access
            output.append(
                "\n%s (see above for traceback): %s\n"
                % (type(self).__name__, self.message)
            )
            return "".join(output)
        else:
            return self.message


OK = error_codes.OK
# tf_export("errors.OK").export_constant(__name__, "OK")
CANCELLED = error_codes.CANCELLED
# tf_export("errors.CANCELLED").export_constant(__name__, "CANCELLED")
UNKNOWN = error_codes.UNKNOWN
# tf_export("errors.UNKNOWN").export_constant(__name__, "UNKNOWN")
INVALID_ARGUMENT = error_codes.INVALID_ARGUMENT
# tf_export("errors.INVALID_ARGUMENT").export_constant(__name__,
# "INVALID_ARGUMENT")
DEADLINE_EXCEEDED = error_codes.DEADLINE_EXCEEDED
# tf_export("errors.DEADLINE_EXCEEDED").export_constant(__name__,
# "DEADLINE_EXCEEDED")
NOT_FOUND = error_codes.NOT_FOUND
# tf_export("errors.NOT_FOUND").export_constant(__name__, "NOT_FOUND")
ALREADY_EXISTS = error_codes.ALREADY_EXISTS
# tf_export("errors.ALREADY_EXISTS").export_constant(__name__, "ALREADY_EXISTS")
PERMISSION_DENIED = error_codes.PERMISSION_DENIED
# tf_export("errors.PERMISSION_DENIED").export_constant(__name__,
# "PERMISSION_DENIED")
UNAUTHENTICATED = error_codes.UNAUTHENTICATED
# tf_export("errors.UNAUTHENTICATED").export_constant(__name__, "UNAUTHENTICATED")
RESOURCE_EXHAUSTED = error_codes.RESOURCE_EXHAUSTED
# tf_export("errors.RESOURCE_EXHAUSTED").export_constant(__name__,
# "RESOURCE_EXHAUSTED")
FAILED_PRECONDITION = error_codes.FAILED_PRECONDITION
# tf_export("errors.FAILED_PRECONDITION").export_constant(__name__,
# "FAILED_PRECONDITION")
ABORTED = error_codes.ABORTED
# tf_export("errors.ABORTED").export_constant(__name__, "ABORTED")
OUT_OF_RANGE = error_codes.OUT_OF_RANGE
# tf_export("errors.OUT_OF_RANGE").export_constant(__name__, "OUT_OF_RANGE")
UNIMPLEMENTED = error_codes.UNIMPLEMENTED
# tf_export("errors.UNIMPLEMENTED").export_constant(__name__, "UNIMPLEMENTED")
INTERNAL = error_codes.INTERNAL
# tf_export("errors.INTERNAL").export_constant(__name__, "INTERNAL")
UNAVAILABLE = error_codes.UNAVAILABLE
# tf_export("errors.UNAVAILABLE").export_constant(__name__, "UNAVAILABLE")
DATA_LOSS = error_codes.DATA_LOSS
# tf_export("errors.DATA_LOSS").export_constant(__name__, "DATA_LOSS")


# @tf_export("errors.CancelledError")
class CancelledError(OpError):
    """Raised when an operation or step is cancelled.

    For example, a long-running operation (e.g.
    @{tf.QueueBase.enqueue} may be
    cancelled by running another operation (e.g.
    @{tf.QueueBase.close},
    or by @{tf.Session.close}.
    A step that is running such a long-running operation will fail by raising
    `CancelledError`.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates a `CancelledError`."""
        super().__init__(node_def, op, message, CANCELLED)


# @tf_export("errors.UnknownError")
class UnknownError(OpError):
    """Unknown error.

    An example of where this error may be returned is if a Status value
    received from another address space belongs to an error-space that
    is not known to this address space. Also errors raised by APIs that
    do not return enough error information may be converted to this
    error.

    @@__init__
    """

    def __init__(self, node_def, op, message, error_code=UNKNOWN):
        """Creates an `UnknownError`."""
        super().__init__(node_def, op, message, error_code)


# @tf_export("errors.InvalidArgumentError")
class InvalidArgumentError(OpError):
    """Raised when an operation receives an invalid argument.

    This may occur, for example, if an operation is receives an input
    tensor that has an invalid value or shape. For example, the
    @{tf.matmul} op will raise this
    error if it receives an input that is not a matrix, and the
    @{tf.reshape} op will raise
    this error if the new shape does not match the number of elements in the input
    tensor.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates an `InvalidArgumentError`."""
        super().__init__(node_def, op, message, INVALID_ARGUMENT)


# @tf_export("errors.DeadlineExceededError")
class DeadlineExceededError(OpError):
    """Raised when a deadline expires before an operation could complete.

    This exception is not currently used.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates a `DeadlineExceededError`."""
        super().__init__(node_def, op, message, DEADLINE_EXCEEDED)


# @tf_export("errors.NotFoundError")
class NotFoundError(OpError):
    """Raised when a requested entity (e.g., a file or directory) was not
    found.

    For example, running the
    @{tf.WholeFileReader.read}
    operation could raise `NotFoundError` if it receives the name of a file that
    does not exist.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates a `NotFoundError`."""
        super().__init__(node_def, op, message, NOT_FOUND)


# @tf_export("errors.AlreadyExistsError")
class AlreadyExistsError(OpError):
    """Raised when an entity that we attempted to create already exists.

    For example, running an operation that saves a file
    (e.g. @{tf.train.Saver.save})
    could potentially raise this exception if an explicit filename for an
    existing file was passed.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates an `AlreadyExistsError`."""
        super().__init__(node_def, op, message, ALREADY_EXISTS)


# @tf_export("errors.PermissionDeniedError")
class PermissionDeniedError(OpError):
    """Raised when the caller does not have permission to run an operation.

    For example, running the
    @{tf.WholeFileReader.read}
    operation could raise `PermissionDeniedError` if it receives the name of a
    file for which the user does not have the read file permission.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates a `PermissionDeniedError`."""
        super().__init__(node_def, op, message, PERMISSION_DENIED)


# @tf_export("errors.UnauthenticatedError")
class UnauthenticatedError(OpError):
    """The request does not have valid authentication credentials.

    This exception is not currently used.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates an `UnauthenticatedError`."""
        super().__init__(node_def, op, message, UNAUTHENTICATED)


# @tf_export("errors.ResourceExhaustedError")
class ResourceExhaustedError(OpError):
    """Some resource has been exhausted.

    For example, this error might be raised if a per-user quota is
    exhausted, or perhaps the entire file system is out of space.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates a `ResourceExhaustedError`."""
        super().__init__(node_def, op, message, RESOURCE_EXHAUSTED)


# @tf_export("errors.FailedPreconditionError")
class FailedPreconditionError(OpError):
    """Operation was rejected because the system is not in a state to execute
    it.

    This exception is most commonly raised when running an operation
    that reads a @{tf.Variable}
    before it has been initialized.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates a `FailedPreconditionError`."""
        super().__init__(node_def, op, message, FAILED_PRECONDITION)


# @tf_export("errors.AbortedError")
class AbortedError(OpError):
    """The operation was aborted, typically due to a concurrent action.

    For example, running a
    @{tf.QueueBase.enqueue}
    operation may raise `AbortedError` if a
    @{tf.QueueBase.close} operation
    previously ran.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates an `AbortedError`."""
        super().__init__(node_def, op, message, ABORTED)


# @tf_export("errors.OutOfRangeError")
class OutOfRangeError(OpError):
    """Raised when an operation iterates past the valid input range.

    This exception is raised in "end-of-file" conditions, such as when a
    @{tf.QueueBase.dequeue}
    operation is blocked on an empty queue, and a
    @{tf.QueueBase.close}
    operation executes.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates an `OutOfRangeError`."""
        super().__init__(node_def, op, message, OUT_OF_RANGE)


# @tf_export("errors.UnimplementedError")
class UnimplementedError(OpError):
    """Raised when an operation has not been implemented.

    Some operations may raise this error when passed otherwise-valid
    arguments that it does not currently support. For example, running
    the @{tf.nn.max_pool} operation
    would raise this error if pooling was requested on the batch dimension,
    because this is not yet supported.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates an `UnimplementedError`."""
        super().__init__(node_def, op, message, UNIMPLEMENTED)


# @tf_export("errors.InternalError")
class InternalError(OpError):
    """Raised when the system experiences an internal error.

    This exception is raised when some invariant expected by the runtime
    has been broken. Catching this exception is not recommended.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates an `InternalError`."""
        super().__init__(node_def, op, message, INTERNAL)


# @tf_export("errors.UnavailableError")
class UnavailableError(OpError):
    """Raised when the runtime is currently unavailable.

    This exception is not currently used.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates an `UnavailableError`."""
        super().__init__(node_def, op, message, UNAVAILABLE)


# @tf_export("errors.DataLossError")
class DataLossError(OpError):
    """Raised when unrecoverable data loss or corruption is encountered.

    For example, this may be raised by running a
    @{tf.WholeFileReader.read}
    operation, if the file is truncated while it is being read.

    @@__init__
    """

    def __init__(self, node_def, op, message):
        """Creates a `DataLossError`."""
        super().__init__(node_def, op, message, DATA_LOSS)


_CODE_TO_EXCEPTION_CLASS = {
    CANCELLED: CancelledError,
    UNKNOWN: UnknownError,
    INVALID_ARGUMENT: InvalidArgumentError,
    DEADLINE_EXCEEDED: DeadlineExceededError,
    NOT_FOUND: NotFoundError,
    ALREADY_EXISTS: AlreadyExistsError,
    PERMISSION_DENIED: PermissionDeniedError,
    UNAUTHENTICATED: UnauthenticatedError,
    RESOURCE_EXHAUSTED: ResourceExhaustedError,
    FAILED_PRECONDITION: FailedPreconditionError,
    ABORTED: AbortedError,
    OUT_OF_RANGE: OutOfRangeError,
    UNIMPLEMENTED: UnimplementedError,
    INTERNAL: InternalError,
    UNAVAILABLE: UnavailableError,
    DATA_LOSS: DataLossError,
}

_EXCEPTION_CLASS_TO_CODE = dict(
    ((class_, code) for (code, class_) in _CODE_TO_EXCEPTION_CLASS.items())
)


# @tf_export("errors.exception_type_from_error_code")
def exception_type_from_error_code(error_code):
    return _CODE_TO_EXCEPTION_CLASS[error_code]


# @tf_export("errors.error_code_from_exception_type")
def error_code_from_exception_type(cls):
    return _EXCEPTION_CLASS_TO_CODE[cls]


def _make_specific_exception(node_def, op, message, error_code):
    try:
        exc_type = exception_type_from_error_code(error_code)
        return exc_type(node_def, op, message)
    except KeyError:
        warnings.warn("Unknown error code: %d" % error_code)
        return UnknownError(node_def, op, message, error_code)


# Named like a function for backwards compatibility with the
# @tf_contextlib.contextmanager version, which was switched to a class to avoid
# some object creation overhead.
# TODO(b/77295559): expand use of TF_Status* SWIG typemap and deprecate this.
# @tf_export("errors.raise_exception_on_not_ok_status")  # pylint: disable=invalid-name
class raise_exception_on_not_ok_status:
    """Context manager to check for C API status."""

    def __enter__(self):
        return "Status not OK"

    def __exit__(self, type_arg, value_arg, traceback_arg):
        return False  # False values do not suppress exceptions



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/allocation_description.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5tensorboard/compat/proto/allocation_description.proto\x12\x0btensorboard\"\xa3\x01\n\x15\x41llocationDescription\x12\x17\n\x0frequested_bytes\x18\x01 \x01(\x03\x12\x17\n\x0f\x61llocated_bytes\x18\x02 \x01(\x03\x12\x16\n\x0e\x61llocator_name\x18\x03 \x01(\t\x12\x15\n\rallocation_id\x18\x04 \x01(\x03\x12\x1c\n\x14has_single_reference\x18\x05 \x01(\x08\x12\x0b\n\x03ptr\x18\x06 \x01(\x04\x42\x9b\x01\n\x18org.tensorflow.frameworkB\x1b\x41llocationDescriptionProtosP\x01Z]github.com/tensorflow/tensorflow/tensorflow/go/core/framework/allocation_description_go_proto\xf8\x01\x01\x62\x06proto3')



_ALLOCATIONDESCRIPTION = DESCRIPTOR.message_types_by_name['AllocationDescription']
AllocationDescription = _reflection.GeneratedProtocolMessageType('AllocationDescription', (_message.Message,), {
  'DESCRIPTOR' : _ALLOCATIONDESCRIPTION,
  '__module__' : 'tensorboard.compat.proto.allocation_description_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.AllocationDescription)
  })
_sym_db.RegisterMessage(AllocationDescription)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030org.tensorflow.frameworkB\033AllocationDescriptionProtosP\001Z]github.com/tensorflow/tensorflow/tensorflow/go/core/framework/allocation_description_go_proto\370\001\001'
  _ALLOCATIONDESCRIPTION._serialized_start=71
  _ALLOCATIONDESCRIPTION._serialized_end=234
# @@protoc_insertion_point(module_scope)



# Copyright 2022 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================

"""Functionality for processing events."""

from typing import Optional

from tensorboard.compat.proto import event_pb2
from tensorboard.util import tb_logging

logger = tb_logging.get_logger()

# Maxmimum length for event writer name.
_MAX_WRITER_NAME_LEN = 128


def ParseFileVersion(file_version: str) -> float:
    """Convert the string file_version in event.proto into a float.

    Args:
      file_version: String file_version from event.proto

    Returns:
      Version number as a float.
    """
    tokens = file_version.split("brain.Event:")
    try:
        return float(tokens[-1])
    except ValueError:
        ## This should never happen according to the definition of file_version
        ## specified in event.proto.
        logger.warning(
            (
                "Invalid event.proto file_version. Defaulting to use of "
                "out-of-order event.step logic for purging expired events."
            )
        )
        return -1


def GetSourceWriter(
    source_metadata: event_pb2.SourceMetadata,
) -> Optional[str]:
    """Gets the source writer name from the source metadata proto."""
    writer_name = source_metadata.writer
    if not writer_name:
        return None
    # Checks the length of the writer name.
    if len(writer_name) > _MAX_WRITER_NAME_LEN:
        logger.error(
            "Source writer name `%s` is too long, maximum allowed length is %d.",
            writer_name,
            _MAX_WRITER_NAME_LEN,
        )
        return None
    return writer_name



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/variable.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'tensorboard/compat/proto/variable.proto\x12\x0btensorboard\"\xcb\x02\n\x0bVariableDef\x12\x15\n\rvariable_name\x18\x01 \x01(\t\x12\x1a\n\x12initial_value_name\x18\x06 \x01(\t\x12\x18\n\x10initializer_name\x18\x02 \x01(\t\x12\x15\n\rsnapshot_name\x18\x03 \x01(\t\x12:\n\x13save_slice_info_def\x18\x04 \x01(\x0b\x32\x1d.tensorboard.SaveSliceInfoDef\x12\x13\n\x0bis_resource\x18\x05 \x01(\x08\x12\x11\n\ttrainable\x18\x07 \x01(\x08\x12=\n\x0fsynchronization\x18\x08 \x01(\x0e\x32$.tensorboard.VariableSynchronization\x12\x35\n\x0b\x61ggregation\x18\t \x01(\x0e\x32 .tensorboard.VariableAggregation\"`\n\x10SaveSliceInfoDef\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x12\n\nfull_shape\x18\x02 \x03(\x03\x12\x12\n\nvar_offset\x18\x03 \x03(\x03\x12\x11\n\tvar_shape\x18\x04 \x03(\x03*\xac\x01\n\x17VariableSynchronization\x12!\n\x1dVARIABLE_SYNCHRONIZATION_AUTO\x10\x00\x12!\n\x1dVARIABLE_SYNCHRONIZATION_NONE\x10\x01\x12%\n!VARIABLE_SYNCHRONIZATION_ON_WRITE\x10\x02\x12$\n VARIABLE_SYNCHRONIZATION_ON_READ\x10\x03*\x9e\x01\n\x13VariableAggregation\x12\x1d\n\x19VARIABLE_AGGREGATION_NONE\x10\x00\x12\x1c\n\x18VARIABLE_AGGREGATION_SUM\x10\x01\x12\x1d\n\x19VARIABLE_AGGREGATION_MEAN\x10\x02\x12+\n\'VARIABLE_AGGREGATION_ONLY_FIRST_REPLICA\x10\x03\x42\x80\x01\n\x18org.tensorflow.frameworkB\x0eVariableProtosP\x01ZOgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/variable_go_proto\xf8\x01\x01\x62\x06proto3')

_VARIABLESYNCHRONIZATION = DESCRIPTOR.enum_types_by_name['VariableSynchronization']
VariableSynchronization = enum_type_wrapper.EnumTypeWrapper(_VARIABLESYNCHRONIZATION)
_VARIABLEAGGREGATION = DESCRIPTOR.enum_types_by_name['VariableAggregation']
VariableAggregation = enum_type_wrapper.EnumTypeWrapper(_VARIABLEAGGREGATION)
VARIABLE_SYNCHRONIZATION_AUTO = 0
VARIABLE_SYNCHRONIZATION_NONE = 1
VARIABLE_SYNCHRONIZATION_ON_WRITE = 2
VARIABLE_SYNCHRONIZATION_ON_READ = 3
VARIABLE_AGGREGATION_NONE = 0
VARIABLE_AGGREGATION_SUM = 1
VARIABLE_AGGREGATION_MEAN = 2
VARIABLE_AGGREGATION_ONLY_FIRST_REPLICA = 3


_VARIABLEDEF = DESCRIPTOR.message_types_by_name['VariableDef']
_SAVESLICEINFODEF = DESCRIPTOR.message_types_by_name['SaveSliceInfoDef']
VariableDef = _reflection.GeneratedProtocolMessageType('VariableDef', (_message.Message,), {
  'DESCRIPTOR' : _VARIABLEDEF,
  '__module__' : 'tensorboard.compat.proto.variable_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.VariableDef)
  })
_sym_db.RegisterMessage(VariableDef)

SaveSliceInfoDef = _reflection.GeneratedProtocolMessageType('SaveSliceInfoDef', (_message.Message,), {
  'DESCRIPTOR' : _SAVESLICEINFODEF,
  '__module__' : 'tensorboard.compat.proto.variable_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.SaveSliceInfoDef)
  })
_sym_db.RegisterMessage(SaveSliceInfoDef)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030org.tensorflow.frameworkB\016VariableProtosP\001ZOgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/variable_go_proto\370\001\001'
  _VARIABLESYNCHRONIZATION._serialized_start=489
  _VARIABLESYNCHRONIZATION._serialized_end=661
  _VARIABLEAGGREGATION._serialized_start=664
  _VARIABLEAGGREGATION._serialized_end=822
  _VARIABLEDEF._serialized_start=57
  _VARIABLEDEF._serialized_end=388
  _SAVESLICEINFODEF._serialized_start=390
  _SAVESLICEINFODEF._serialized_end=486
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/uploader/proto/tensor.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tensorboard.compat.proto import summary_pb2 as tensorboard_dot_compat_dot_proto_dot_summary__pb2
try:
  tensorboard_dot_compat_dot_proto_dot_histogram__pb2 = tensorboard_dot_compat_dot_proto_dot_summary__pb2.tensorboard_dot_compat_dot_proto_dot_histogram__pb2
except AttributeError:
  tensorboard_dot_compat_dot_proto_dot_histogram__pb2 = tensorboard_dot_compat_dot_proto_dot_summary__pb2.tensorboard.compat.proto.histogram_pb2
from tensorboard.compat.proto import tensor_pb2 as tensorboard_dot_compat_dot_proto_dot_tensor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'tensorboard/uploader/proto/tensor.proto\x12\x13tensorboard.service\x1a\x1fgoogle/protobuf/timestamp.proto\x1a&tensorboard/compat/proto/summary.proto\x1a%tensorboard/compat/proto/tensor.proto\"s\n\x0bTensorPoint\x12\x0c\n\x04step\x18\x01 \x01(\x03\x12-\n\twall_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x05value\x18\x03 \x01(\x0b\x32\x18.tensorboard.TensorProto\"\x88\x01\n\tTensorTag\x12\x10\n\x08max_step\x18\x01 \x01(\x03\x12\x31\n\rmax_wall_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x36\n\x10summary_metadata\x18\x03 \x01(\x0b\x32\x1c.tensorboard.SummaryMetadatab\x06proto3')



_TENSORPOINT = DESCRIPTOR.message_types_by_name['TensorPoint']
_TENSORTAG = DESCRIPTOR.message_types_by_name['TensorTag']
TensorPoint = _reflection.GeneratedProtocolMessageType('TensorPoint', (_message.Message,), {
  'DESCRIPTOR' : _TENSORPOINT,
  '__module__' : 'tensorboard.uploader.proto.tensor_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.TensorPoint)
  })
_sym_db.RegisterMessage(TensorPoint)

TensorTag = _reflection.GeneratedProtocolMessageType('TensorTag', (_message.Message,), {
  'DESCRIPTOR' : _TENSORTAG,
  '__module__' : 'tensorboard.uploader.proto.tensor_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.TensorTag)
  })
_sym_db.RegisterMessage(TensorTag)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TENSORPOINT._serialized_start=176
  _TENSORPOINT._serialized_end=291
  _TENSORTAG._serialized_start=294
  _TENSORTAG._serialized_end=430
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/uploader/proto/write_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tensorboard.uploader.proto import experiment_pb2 as tensorboard_dot_uploader_dot_proto_dot_experiment__pb2
from tensorboard.uploader.proto import scalar_pb2 as tensorboard_dot_uploader_dot_proto_dot_scalar__pb2
from tensorboard.uploader.proto import tensor_pb2 as tensorboard_dot_uploader_dot_proto_dot_tensor__pb2
from tensorboard.uploader.proto import blob_pb2 as tensorboard_dot_uploader_dot_proto_dot_blob__pb2
from tensorboard.compat.proto import summary_pb2 as tensorboard_dot_compat_dot_proto_dot_summary__pb2
try:
  tensorboard_dot_compat_dot_proto_dot_histogram__pb2 = tensorboard_dot_compat_dot_proto_dot_summary__pb2.tensorboard_dot_compat_dot_proto_dot_histogram__pb2
except AttributeError:
  tensorboard_dot_compat_dot_proto_dot_histogram__pb2 = tensorboard_dot_compat_dot_proto_dot_summary__pb2.tensorboard.compat.proto.histogram_pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.tensorboard/uploader/proto/write_service.proto\x12\x13tensorboard.service\x1a\x1fgoogle/protobuf/timestamp.proto\x1a+tensorboard/uploader/proto/experiment.proto\x1a\'tensorboard/uploader/proto/scalar.proto\x1a\'tensorboard/uploader/proto/tensor.proto\x1a%tensorboard/uploader/proto/blob.proto\x1a&tensorboard/compat/proto/summary.proto\"<\n\x17\x43reateExperimentRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\">\n\x18\x43reateExperimentResponse\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t\"\x8c\x01\n\x17UpdateExperimentRequest\x12\x33\n\nexperiment\x18\x01 \x01(\x0b\x32\x1f.tensorboard.service.Experiment\x12<\n\x0f\x65xperiment_mask\x18\x02 \x01(\x0b\x32#.tensorboard.service.ExperimentMask\"\x1a\n\x18UpdateExperimentResponse\"0\n\x17\x44\x65leteExperimentRequest\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\"\x1a\n\x18\x44\x65leteExperimentResponse\"\'\n\x10PurgeDataRequest\x12\x13\n\x0b\x62\x61tch_limit\x18\x01 \x01(\x05\"I\n\x11PurgeDataResponse\x12\x34\n\x0bpurge_stats\x18\x01 \x01(\x0b\x32\x1f.tensorboard.service.PurgeStats\">\n\nPurgeStats\x12\x0c\n\x04tags\x18\x01 \x01(\x05\x12\x13\n\x0b\x65xperiments\x18\x02 \x01(\x05\x12\r\n\x05users\x18\x03 \x01(\x05\"\xad\x02\n\x12WriteScalarRequest\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x39\n\x04runs\x18\x02 \x03(\x0b\x32+.tensorboard.service.WriteScalarRequest.Run\x1aN\n\x03Run\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x39\n\x04tags\x18\x02 \x03(\x0b\x32+.tensorboard.service.WriteScalarRequest.Tag\x1au\n\x03Tag\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x30\n\x06points\x18\x02 \x03(\x0b\x32 .tensorboard.service.ScalarPoint\x12.\n\x08metadata\x18\x03 \x01(\x0b\x32\x1c.tensorboard.SummaryMetadata\"\x15\n\x13WriteScalarResponse\"\xad\x02\n\x12WriteTensorRequest\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x39\n\x04runs\x18\x02 \x03(\x0b\x32+.tensorboard.service.WriteTensorRequest.Run\x1aN\n\x03Run\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x39\n\x04tags\x18\x02 \x03(\x0b\x32+.tensorboard.service.WriteTensorRequest.Tag\x1au\n\x03Tag\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x30\n\x06points\x18\x02 \x03(\x0b\x32 .tensorboard.service.TensorPoint\x12.\n\x08metadata\x18\x03 \x01(\x0b\x32\x1c.tensorboard.SummaryMetadata\"\x15\n\x13WriteTensorResponse\"\xdd\x01\n\x1eGetOrCreateBlobSequenceRequest\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0b\n\x03run\x18\x02 \x01(\t\x12\x0b\n\x03tag\x18\x03 \x01(\t\x12\x0c\n\x04step\x18\x04 \x01(\x03\x12-\n\twall_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1d\n\x15\x66inal_sequence_length\x18\x06 \x01(\x03\x12.\n\x08metadata\x18\x07 \x01(\x0b\x32\x1c.tensorboard.SummaryMetadata\";\n\x1fGetOrCreateBlobSequenceResponse\x12\x18\n\x10\x62lob_sequence_id\x18\x01 \x01(\t\"A\n\x16GetBlobMetadataRequest\x12\x18\n\x10\x62lob_sequence_id\x18\x01 \x01(\t\x12\r\n\x05index\x18\x02 \x01(\x03\"k\n\x17GetBlobMetadataResponse\x12\x32\n\nblob_state\x18\x01 \x01(\x0e\x32\x1e.tensorboard.service.BlobState\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x0e\n\x06\x63rc32c\x18\x03 \x01(\x07\"\xac\x01\n\x10WriteBlobRequest\x12\x18\n\x10\x62lob_sequence_id\x18\x01 \x01(\t\x12\r\n\x05index\x18\x02 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\x12\x0e\n\x06offset\x18\x04 \x01(\x03\x12\x0e\n\x06\x63rc32c\x18\x05 \x01(\x07\x12\x17\n\x0f\x66inalize_object\x18\x06 \x01(\x08\x12\x14\n\x0c\x66inal_crc32c\x18\x07 \x01(\x07\x12\x12\n\nblob_bytes\x18\x08 \x01(\x03\"U\n\x11WriteBlobResponse\x12\x32\n\nblob_state\x18\x01 \x01(\x0e\x32\x1e.tensorboard.service.BlobState\x12\x0c\n\x04size\x18\x02 \x01(\x03\"\x16\n\x14\x44\x65leteOwnUserRequest\"\x17\n\x15\x44\x65leteOwnUserResponse2\xde\x08\n\x18TensorBoardWriterService\x12q\n\x10\x43reateExperiment\x12,.tensorboard.service.CreateExperimentRequest\x1a-.tensorboard.service.CreateExperimentResponse\"\x00\x12q\n\x10UpdateExperiment\x12,.tensorboard.service.UpdateExperimentRequest\x1a-.tensorboard.service.UpdateExperimentResponse\"\x00\x12q\n\x10\x44\x65leteExperiment\x12,.tensorboard.service.DeleteExperimentRequest\x1a-.tensorboard.service.DeleteExperimentResponse\"\x00\x12\\\n\tPurgeData\x12%.tensorboard.service.PurgeDataRequest\x1a&.tensorboard.service.PurgeDataResponse\"\x00\x12\x62\n\x0bWriteScalar\x12\'.tensorboard.service.WriteScalarRequest\x1a(.tensorboard.service.WriteScalarResponse\"\x00\x12\x62\n\x0bWriteTensor\x12\'.tensorboard.service.WriteTensorRequest\x1a(.tensorboard.service.WriteTensorResponse\"\x00\x12\x86\x01\n\x17GetOrCreateBlobSequence\x12\x33.tensorboard.service.GetOrCreateBlobSequenceRequest\x1a\x34.tensorboard.service.GetOrCreateBlobSequenceResponse\"\x00\x12n\n\x0fGetBlobMetadata\x12+.tensorboard.service.GetBlobMetadataRequest\x1a,.tensorboard.service.GetBlobMetadataResponse\"\x00\x12`\n\tWriteBlob\x12%.tensorboard.service.WriteBlobRequest\x1a&.tensorboard.service.WriteBlobResponse\"\x00(\x01\x30\x01\x12h\n\rDeleteOwnUser\x12).tensorboard.service.DeleteOwnUserRequest\x1a*.tensorboard.service.DeleteOwnUserResponse\"\x00\x62\x06proto3')



_CREATEEXPERIMENTREQUEST = DESCRIPTOR.message_types_by_name['CreateExperimentRequest']
_CREATEEXPERIMENTRESPONSE = DESCRIPTOR.message_types_by_name['CreateExperimentResponse']
_UPDATEEXPERIMENTREQUEST = DESCRIPTOR.message_types_by_name['UpdateExperimentRequest']
_UPDATEEXPERIMENTRESPONSE = DESCRIPTOR.message_types_by_name['UpdateExperimentResponse']
_DELETEEXPERIMENTREQUEST = DESCRIPTOR.message_types_by_name['DeleteExperimentRequest']
_DELETEEXPERIMENTRESPONSE = DESCRIPTOR.message_types_by_name['DeleteExperimentResponse']
_PURGEDATAREQUEST = DESCRIPTOR.message_types_by_name['PurgeDataRequest']
_PURGEDATARESPONSE = DESCRIPTOR.message_types_by_name['PurgeDataResponse']
_PURGESTATS = DESCRIPTOR.message_types_by_name['PurgeStats']
_WRITESCALARREQUEST = DESCRIPTOR.message_types_by_name['WriteScalarRequest']
_WRITESCALARREQUEST_RUN = _WRITESCALARREQUEST.nested_types_by_name['Run']
_WRITESCALARREQUEST_TAG = _WRITESCALARREQUEST.nested_types_by_name['Tag']
_WRITESCALARRESPONSE = DESCRIPTOR.message_types_by_name['WriteScalarResponse']
_WRITETENSORREQUEST = DESCRIPTOR.message_types_by_name['WriteTensorRequest']
_WRITETENSORREQUEST_RUN = _WRITETENSORREQUEST.nested_types_by_name['Run']
_WRITETENSORREQUEST_TAG = _WRITETENSORREQUEST.nested_types_by_name['Tag']
_WRITETENSORRESPONSE = DESCRIPTOR.message_types_by_name['WriteTensorResponse']
_GETORCREATEBLOBSEQUENCEREQUEST = DESCRIPTOR.message_types_by_name['GetOrCreateBlobSequenceRequest']
_GETORCREATEBLOBSEQUENCERESPONSE = DESCRIPTOR.message_types_by_name['GetOrCreateBlobSequenceResponse']
_GETBLOBMETADATAREQUEST = DESCRIPTOR.message_types_by_name['GetBlobMetadataRequest']
_GETBLOBMETADATARESPONSE = DESCRIPTOR.message_types_by_name['GetBlobMetadataResponse']
_WRITEBLOBREQUEST = DESCRIPTOR.message_types_by_name['WriteBlobRequest']
_WRITEBLOBRESPONSE = DESCRIPTOR.message_types_by_name['WriteBlobResponse']
_DELETEOWNUSERREQUEST = DESCRIPTOR.message_types_by_name['DeleteOwnUserRequest']
_DELETEOWNUSERRESPONSE = DESCRIPTOR.message_types_by_name['DeleteOwnUserResponse']
CreateExperimentRequest = _reflection.GeneratedProtocolMessageType('CreateExperimentRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEEXPERIMENTREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.CreateExperimentRequest)
  })
_sym_db.RegisterMessage(CreateExperimentRequest)

CreateExperimentResponse = _reflection.GeneratedProtocolMessageType('CreateExperimentResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEEXPERIMENTRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.CreateExperimentResponse)
  })
_sym_db.RegisterMessage(CreateExperimentResponse)

UpdateExperimentRequest = _reflection.GeneratedProtocolMessageType('UpdateExperimentRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEEXPERIMENTREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.UpdateExperimentRequest)
  })
_sym_db.RegisterMessage(UpdateExperimentRequest)

UpdateExperimentResponse = _reflection.GeneratedProtocolMessageType('UpdateExperimentResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEEXPERIMENTRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.UpdateExperimentResponse)
  })
_sym_db.RegisterMessage(UpdateExperimentResponse)

DeleteExperimentRequest = _reflection.GeneratedProtocolMessageType('DeleteExperimentRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEEXPERIMENTREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.DeleteExperimentRequest)
  })
_sym_db.RegisterMessage(DeleteExperimentRequest)

DeleteExperimentResponse = _reflection.GeneratedProtocolMessageType('DeleteExperimentResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEEXPERIMENTRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.DeleteExperimentResponse)
  })
_sym_db.RegisterMessage(DeleteExperimentResponse)

PurgeDataRequest = _reflection.GeneratedProtocolMessageType('PurgeDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _PURGEDATAREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.PurgeDataRequest)
  })
_sym_db.RegisterMessage(PurgeDataRequest)

PurgeDataResponse = _reflection.GeneratedProtocolMessageType('PurgeDataResponse', (_message.Message,), {
  'DESCRIPTOR' : _PURGEDATARESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.PurgeDataResponse)
  })
_sym_db.RegisterMessage(PurgeDataResponse)

PurgeStats = _reflection.GeneratedProtocolMessageType('PurgeStats', (_message.Message,), {
  'DESCRIPTOR' : _PURGESTATS,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.PurgeStats)
  })
_sym_db.RegisterMessage(PurgeStats)

WriteScalarRequest = _reflection.GeneratedProtocolMessageType('WriteScalarRequest', (_message.Message,), {

  'Run' : _reflection.GeneratedProtocolMessageType('Run', (_message.Message,), {
    'DESCRIPTOR' : _WRITESCALARREQUEST_RUN,
    '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.WriteScalarRequest.Run)
    })
  ,

  'Tag' : _reflection.GeneratedProtocolMessageType('Tag', (_message.Message,), {
    'DESCRIPTOR' : _WRITESCALARREQUEST_TAG,
    '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.WriteScalarRequest.Tag)
    })
  ,
  'DESCRIPTOR' : _WRITESCALARREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteScalarRequest)
  })
_sym_db.RegisterMessage(WriteScalarRequest)
_sym_db.RegisterMessage(WriteScalarRequest.Run)
_sym_db.RegisterMessage(WriteScalarRequest.Tag)

WriteScalarResponse = _reflection.GeneratedProtocolMessageType('WriteScalarResponse', (_message.Message,), {
  'DESCRIPTOR' : _WRITESCALARRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteScalarResponse)
  })
_sym_db.RegisterMessage(WriteScalarResponse)

WriteTensorRequest = _reflection.GeneratedProtocolMessageType('WriteTensorRequest', (_message.Message,), {

  'Run' : _reflection.GeneratedProtocolMessageType('Run', (_message.Message,), {
    'DESCRIPTOR' : _WRITETENSORREQUEST_RUN,
    '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.WriteTensorRequest.Run)
    })
  ,

  'Tag' : _reflection.GeneratedProtocolMessageType('Tag', (_message.Message,), {
    'DESCRIPTOR' : _WRITETENSORREQUEST_TAG,
    '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.WriteTensorRequest.Tag)
    })
  ,
  'DESCRIPTOR' : _WRITETENSORREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteTensorRequest)
  })
_sym_db.RegisterMessage(WriteTensorRequest)
_sym_db.RegisterMessage(WriteTensorRequest.Run)
_sym_db.RegisterMessage(WriteTensorRequest.Tag)

WriteTensorResponse = _reflection.GeneratedProtocolMessageType('WriteTensorResponse', (_message.Message,), {
  'DESCRIPTOR' : _WRITETENSORRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteTensorResponse)
  })
_sym_db.RegisterMessage(WriteTensorResponse)

GetOrCreateBlobSequenceRequest = _reflection.GeneratedProtocolMessageType('GetOrCreateBlobSequenceRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETORCREATEBLOBSEQUENCEREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.GetOrCreateBlobSequenceRequest)
  })
_sym_db.RegisterMessage(GetOrCreateBlobSequenceRequest)

GetOrCreateBlobSequenceResponse = _reflection.GeneratedProtocolMessageType('GetOrCreateBlobSequenceResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETORCREATEBLOBSEQUENCERESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.GetOrCreateBlobSequenceResponse)
  })
_sym_db.RegisterMessage(GetOrCreateBlobSequenceResponse)

GetBlobMetadataRequest = _reflection.GeneratedProtocolMessageType('GetBlobMetadataRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETBLOBMETADATAREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.GetBlobMetadataRequest)
  })
_sym_db.RegisterMessage(GetBlobMetadataRequest)

GetBlobMetadataResponse = _reflection.GeneratedProtocolMessageType('GetBlobMetadataResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETBLOBMETADATARESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.GetBlobMetadataResponse)
  })
_sym_db.RegisterMessage(GetBlobMetadataResponse)

WriteBlobRequest = _reflection.GeneratedProtocolMessageType('WriteBlobRequest', (_message.Message,), {
  'DESCRIPTOR' : _WRITEBLOBREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteBlobRequest)
  })
_sym_db.RegisterMessage(WriteBlobRequest)

WriteBlobResponse = _reflection.GeneratedProtocolMessageType('WriteBlobResponse', (_message.Message,), {
  'DESCRIPTOR' : _WRITEBLOBRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteBlobResponse)
  })
_sym_db.RegisterMessage(WriteBlobResponse)

DeleteOwnUserRequest = _reflection.GeneratedProtocolMessageType('DeleteOwnUserRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEOWNUSERREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.DeleteOwnUserRequest)
  })
_sym_db.RegisterMessage(DeleteOwnUserRequest)

DeleteOwnUserResponse = _reflection.GeneratedProtocolMessageType('DeleteOwnUserResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEOWNUSERRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.DeleteOwnUserResponse)
  })
_sym_db.RegisterMessage(DeleteOwnUserResponse)

_TENSORBOARDWRITERSERVICE = DESCRIPTOR.services_by_name['TensorBoardWriterService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CREATEEXPERIMENTREQUEST._serialized_start=310
  _CREATEEXPERIMENTREQUEST._serialized_end=370
  _CREATEEXPERIMENTRESPONSE._serialized_start=372
  _CREATEEXPERIMENTRESPONSE._serialized_end=434
  _UPDATEEXPERIMENTREQUEST._serialized_start=437
  _UPDATEEXPERIMENTREQUEST._serialized_end=577
  _UPDATEEXPERIMENTRESPONSE._serialized_start=579
  _UPDATEEXPERIMENTRESPONSE._serialized_end=605
  _DELETEEXPERIMENTREQUEST._serialized_start=607
  _DELETEEXPERIMENTREQUEST._serialized_end=655
  _DELETEEXPERIMENTRESPONSE._serialized_start=657
  _DELETEEXPERIMENTRESPONSE._serialized_end=683
  _PURGEDATAREQUEST._serialized_start=685
  _PURGEDATAREQUEST._serialized_end=724
  _PURGEDATARESPONSE._serialized_start=726
  _PURGEDATARESPONSE._serialized_end=799
  _PURGESTATS._serialized_start=801
  _PURGESTATS._serialized_end=863
  _WRITESCALARREQUEST._serialized_start=866
  _WRITESCALARREQUEST._serialized_end=1167
  _WRITESCALARREQUEST_RUN._serialized_start=970
  _WRITESCALARREQUEST_RUN._serialized_end=1048
  _WRITESCALARREQUEST_TAG._serialized_start=1050
  _WRITESCALARREQUEST_TAG._serialized_end=1167
  _WRITESCALARRESPONSE._serialized_start=1169
  _WRITESCALARRESPONSE._serialized_end=1190
  _WRITETENSORREQUEST._serialized_start=1193
  _WRITETENSORREQUEST._serialized_end=1494
  _WRITETENSORREQUEST_RUN._serialized_start=1297
  _WRITETENSORREQUEST_RUN._serialized_end=1375
  _WRITETENSORREQUEST_TAG._serialized_start=1377
  _WRITETENSORREQUEST_TAG._serialized_end=1494
  _WRITETENSORRESPONSE._serialized_start=1496
  _WRITETENSORRESPONSE._serialized_end=1517
  _GETORCREATEBLOBSEQUENCEREQUEST._serialized_start=1520
  _GETORCREATEBLOBSEQUENCEREQUEST._serialized_end=1741
  _GETORCREATEBLOBSEQUENCERESPONSE._serialized_start=1743
  _GETORCREATEBLOBSEQUENCERESPONSE._serialized_end=1802
  _GETBLOBMETADATAREQUEST._serialized_start=1804
  _GETBLOBMETADATAREQUEST._serialized_end=1869
  _GETBLOBMETADATARESPONSE._serialized_start=1871
  _GETBLOBMETADATARESPONSE._serialized_end=1978
  _WRITEBLOBREQUEST._serialized_start=1981
  _WRITEBLOBREQUEST._serialized_end=2153
  _WRITEBLOBRESPONSE._serialized_start=2155
  _WRITEBLOBRESPONSE._serialized_end=2240
  _DELETEOWNUSERREQUEST._serialized_start=2242
  _DELETEOWNUSERREQUEST._serialized_end=2264
  _DELETEOWNUSERRESPONSE._serialized_start=2266
  _DELETEOWNUSERRESPONSE._serialized_end=2289
  _TENSORBOARDWRITERSERVICE._serialized_start=2292
  _TENSORBOARDWRITERSERVICE._serialized_end=3410
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/tensor.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorboard.compat.proto import resource_handle_pb2 as tensorboard_dot_compat_dot_proto_dot_resource__handle__pb2
from tensorboard.compat.proto import tensor_shape_pb2 as tensorboard_dot_compat_dot_proto_dot_tensor__shape__pb2
from tensorboard.compat.proto import types_pb2 as tensorboard_dot_compat_dot_proto_dot_types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%tensorboard/compat/proto/tensor.proto\x12\x0btensorboard\x1a.tensorboard/compat/proto/resource_handle.proto\x1a+tensorboard/compat/proto/tensor_shape.proto\x1a$tensorboard/compat/proto/types.proto\"\xa4\x04\n\x0bTensorProto\x12$\n\x05\x64type\x18\x01 \x01(\x0e\x32\x15.tensorboard.DataType\x12\x33\n\x0ctensor_shape\x18\x02 \x01(\x0b\x32\x1d.tensorboard.TensorShapeProto\x12\x16\n\x0eversion_number\x18\x03 \x01(\x05\x12\x16\n\x0etensor_content\x18\x04 \x01(\x0c\x12\x14\n\x08half_val\x18\r \x03(\x05\x42\x02\x10\x01\x12\x15\n\tfloat_val\x18\x05 \x03(\x02\x42\x02\x10\x01\x12\x16\n\ndouble_val\x18\x06 \x03(\x01\x42\x02\x10\x01\x12\x13\n\x07int_val\x18\x07 \x03(\x05\x42\x02\x10\x01\x12\x12\n\nstring_val\x18\x08 \x03(\x0c\x12\x18\n\x0cscomplex_val\x18\t \x03(\x02\x42\x02\x10\x01\x12\x15\n\tint64_val\x18\n \x03(\x03\x42\x02\x10\x01\x12\x14\n\x08\x62ool_val\x18\x0b \x03(\x08\x42\x02\x10\x01\x12\x18\n\x0c\x64\x63omplex_val\x18\x0c \x03(\x01\x42\x02\x10\x01\x12=\n\x13resource_handle_val\x18\x0e \x03(\x0b\x32 .tensorboard.ResourceHandleProto\x12\x38\n\x0bvariant_val\x18\x0f \x03(\x0b\x32#.tensorboard.VariantTensorDataProto\x12\x16\n\nuint32_val\x18\x10 \x03(\rB\x02\x10\x01\x12\x16\n\nuint64_val\x18\x11 \x03(\x04\x42\x02\x10\x01\x12\x12\n\nfloat8_val\x18\x12 \x01(\x0c\"h\n\x16VariantTensorDataProto\x12\x11\n\ttype_name\x18\x01 \x01(\t\x12\x10\n\x08metadata\x18\x02 \x01(\x0c\x12)\n\x07tensors\x18\x03 \x03(\x0b\x32\x18.tensorboard.TensorProtoB|\n\x18org.tensorflow.frameworkB\x0cTensorProtosP\x01ZMgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/tensor_go_proto\xf8\x01\x01\x62\x06proto3')



_TENSORPROTO = DESCRIPTOR.message_types_by_name['TensorProto']
_VARIANTTENSORDATAPROTO = DESCRIPTOR.message_types_by_name['VariantTensorDataProto']
TensorProto = _reflection.GeneratedProtocolMessageType('TensorProto', (_message.Message,), {
  'DESCRIPTOR' : _TENSORPROTO,
  '__module__' : 'tensorboard.compat.proto.tensor_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.TensorProto)
  })
_sym_db.RegisterMessage(TensorProto)

VariantTensorDataProto = _reflection.GeneratedProtocolMessageType('VariantTensorDataProto', (_message.Message,), {
  'DESCRIPTOR' : _VARIANTTENSORDATAPROTO,
  '__module__' : 'tensorboard.compat.proto.tensor_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.VariantTensorDataProto)
  })
_sym_db.RegisterMessage(VariantTensorDataProto)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030org.tensorflow.frameworkB\014TensorProtosP\001ZMgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/tensor_go_proto\370\001\001'
  _TENSORPROTO.fields_by_name['half_val']._options = None
  _TENSORPROTO.fields_by_name['half_val']._serialized_options = b'\020\001'
  _TENSORPROTO.fields_by_name['float_val']._options = None
  _TENSORPROTO.fields_by_name['float_val']._serialized_options = b'\020\001'
  _TENSORPROTO.fields_by_name['double_val']._options = None
  _TENSORPROTO.fields_by_name['double_val']._serialized_options = b'\020\001'
  _TENSORPROTO.fields_by_name['int_val']._options = None
  _TENSORPROTO.fields_by_name['int_val']._serialized_options = b'\020\001'
  _TENSORPROTO.fields_by_name['scomplex_val']._options = None
  _TENSORPROTO.fields_by_name['scomplex_val']._serialized_options = b'\020\001'
  _TENSORPROTO.fields_by_name['int64_val']._options = None
  _TENSORPROTO.fields_by_name['int64_val']._serialized_options = b'\020\001'
  _TENSORPROTO.fields_by_name['bool_val']._options = None
  _TENSORPROTO.fields_by_name['bool_val']._serialized_options = b'\020\001'
  _TENSORPROTO.fields_by_name['dcomplex_val']._options = None
  _TENSORPROTO.fields_by_name['dcomplex_val']._serialized_options = b'\020\001'
  _TENSORPROTO.fields_by_name['uint32_val']._options = None
  _TENSORPROTO.fields_by_name['uint32_val']._serialized_options = b'\020\001'
  _TENSORPROTO.fields_by_name['uint64_val']._options = None
  _TENSORPROTO.fields_by_name['uint64_val']._serialized_options = b'\020\001'
  _TENSORPROTO._serialized_start=186
  _TENSORPROTO._serialized_end=734
  _VARIANTTENSORDATAPROTO._serialized_start=736
  _VARIANTTENSORDATAPROTO._serialized_end=840
# @@protoc_insertion_point(module_scope)



# Copyright 2018 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================


from tensorboard.compat.proto.config_pb2 import *  # noqa
from tensorboard.compat.proto.event_pb2 import *  # noqa
from tensorboard.compat.proto.graph_pb2 import *  # noqa
from tensorboard.compat.proto.meta_graph_pb2 import *  # noqa
from tensorboard.compat.proto.summary_pb2 import *  # noqa
from .dtypes import as_dtype  # noqa
from .dtypes import DType  # noqa
from .dtypes import string  # noqa
from . import app  # noqa
from . import compat  # noqa
from . import dtypes  # noqa
from . import error_codes  # noqa
from . import errors  # noqa
from . import flags  # noqa
from . import io  # noqa
from . import pywrap_tensorflow  # noqa
from . import tensor_shape  # noqa

compat.v1.errors = errors

# Set a fake __version__ to help distinguish this as our own stub API.
__version__ = "stub"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/struct.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorboard.compat.proto import tensor_pb2 as tensorboard_dot_compat_dot_proto_dot_tensor__pb2
from tensorboard.compat.proto import tensor_shape_pb2 as tensorboard_dot_compat_dot_proto_dot_tensor__shape__pb2
from tensorboard.compat.proto import types_pb2 as tensorboard_dot_compat_dot_proto_dot_types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%tensorboard/compat/proto/struct.proto\x12\x0btensorboard\x1a%tensorboard/compat/proto/tensor.proto\x1a+tensorboard/compat/proto/tensor_shape.proto\x1a$tensorboard/compat/proto/types.proto\"\x9a\x05\n\x0fStructuredValue\x12,\n\nnone_value\x18\x01 \x01(\x0b\x32\x16.tensorboard.NoneValueH\x00\x12\x17\n\rfloat64_value\x18\x0b \x01(\x01H\x00\x12\x15\n\x0bint64_value\x18\x0c \x01(\x12H\x00\x12\x16\n\x0cstring_value\x18\r \x01(\tH\x00\x12\x14\n\nbool_value\x18\x0e \x01(\x08H\x00\x12;\n\x12tensor_shape_value\x18\x1f \x01(\x0b\x32\x1d.tensorboard.TensorShapeProtoH\x00\x12\x33\n\x12tensor_dtype_value\x18  \x01(\x0e\x32\x15.tensorboard.DataTypeH\x00\x12\x39\n\x11tensor_spec_value\x18! \x01(\x0b\x32\x1c.tensorboard.TensorSpecProtoH\x00\x12\x35\n\x0ftype_spec_value\x18\" \x01(\x0b\x32\x1a.tensorboard.TypeSpecProtoH\x00\x12H\n\x19\x62ounded_tensor_spec_value\x18# \x01(\x0b\x32#.tensorboard.BoundedTensorSpecProtoH\x00\x12,\n\nlist_value\x18\x33 \x01(\x0b\x32\x16.tensorboard.ListValueH\x00\x12.\n\x0btuple_value\x18\x34 \x01(\x0b\x32\x17.tensorboard.TupleValueH\x00\x12,\n\ndict_value\x18\x35 \x01(\x0b\x32\x16.tensorboard.DictValueH\x00\x12\x39\n\x11named_tuple_value\x18\x36 \x01(\x0b\x32\x1c.tensorboard.NamedTupleValueH\x00\x42\x06\n\x04kind\"\x0b\n\tNoneValue\"9\n\tListValue\x12,\n\x06values\x18\x01 \x03(\x0b\x32\x1c.tensorboard.StructuredValue\":\n\nTupleValue\x12,\n\x06values\x18\x01 \x03(\x0b\x32\x1c.tensorboard.StructuredValue\"\x8c\x01\n\tDictValue\x12\x32\n\x06\x66ields\x18\x01 \x03(\x0b\x32\".tensorboard.DictValue.FieldsEntry\x1aK\n\x0b\x46ieldsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12+\n\x05value\x18\x02 \x01(\x0b\x32\x1c.tensorboard.StructuredValue:\x02\x38\x01\"E\n\tPairValue\x12\x0b\n\x03key\x18\x01 \x01(\t\x12+\n\x05value\x18\x02 \x01(\x0b\x32\x1c.tensorboard.StructuredValue\"G\n\x0fNamedTupleValue\x12\x0c\n\x04name\x18\x01 \x01(\t\x12&\n\x06values\x18\x02 \x03(\x0b\x32\x16.tensorboard.PairValue\"s\n\x0fTensorSpecProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12,\n\x05shape\x18\x02 \x01(\x0b\x32\x1d.tensorboard.TensorShapeProto\x12$\n\x05\x64type\x18\x03 \x01(\x0e\x32\x15.tensorboard.DataType\"\xd0\x01\n\x16\x42oundedTensorSpecProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12,\n\x05shape\x18\x02 \x01(\x0b\x32\x1d.tensorboard.TensorShapeProto\x12$\n\x05\x64type\x18\x03 \x01(\x0e\x32\x15.tensorboard.DataType\x12)\n\x07minimum\x18\x04 \x01(\x0b\x32\x18.tensorboard.TensorProto\x12)\n\x07maximum\x18\x05 \x01(\x0b\x32\x18.tensorboard.TensorProto\"\xfa\x03\n\rTypeSpecProto\x12\x41\n\x0ftype_spec_class\x18\x01 \x01(\x0e\x32(.tensorboard.TypeSpecProto.TypeSpecClass\x12\x30\n\ntype_state\x18\x02 \x01(\x0b\x32\x1c.tensorboard.StructuredValue\x12\x1c\n\x14type_spec_class_name\x18\x03 \x01(\t\x12\x1b\n\x13num_flat_components\x18\x04 \x01(\x05\"\xb8\x02\n\rTypeSpecClass\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x16\n\x12SPARSE_TENSOR_SPEC\x10\x01\x12\x17\n\x13INDEXED_SLICES_SPEC\x10\x02\x12\x16\n\x12RAGGED_TENSOR_SPEC\x10\x03\x12\x15\n\x11TENSOR_ARRAY_SPEC\x10\x04\x12\x15\n\x11\x44\x41TA_DATASET_SPEC\x10\x05\x12\x16\n\x12\x44\x41TA_ITERATOR_SPEC\x10\x06\x12\x11\n\rOPTIONAL_SPEC\x10\x07\x12\x14\n\x10PER_REPLICA_SPEC\x10\x08\x12\x11\n\rVARIABLE_SPEC\x10\t\x12\x16\n\x12ROW_PARTITION_SPEC\x10\n\x12\x18\n\x14REGISTERED_TYPE_SPEC\x10\x0c\x12\x17\n\x13\x45XTENSION_TYPE_SPEC\x10\r\"\x04\x08\x0b\x10\x0b\x42WZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_protob\x06proto3')



_STRUCTUREDVALUE = DESCRIPTOR.message_types_by_name['StructuredValue']
_NONEVALUE = DESCRIPTOR.message_types_by_name['NoneValue']
_LISTVALUE = DESCRIPTOR.message_types_by_name['ListValue']
_TUPLEVALUE = DESCRIPTOR.message_types_by_name['TupleValue']
_DICTVALUE = DESCRIPTOR.message_types_by_name['DictValue']
_DICTVALUE_FIELDSENTRY = _DICTVALUE.nested_types_by_name['FieldsEntry']
_PAIRVALUE = DESCRIPTOR.message_types_by_name['PairValue']
_NAMEDTUPLEVALUE = DESCRIPTOR.message_types_by_name['NamedTupleValue']
_TENSORSPECPROTO = DESCRIPTOR.message_types_by_name['TensorSpecProto']
_BOUNDEDTENSORSPECPROTO = DESCRIPTOR.message_types_by_name['BoundedTensorSpecProto']
_TYPESPECPROTO = DESCRIPTOR.message_types_by_name['TypeSpecProto']
_TYPESPECPROTO_TYPESPECCLASS = _TYPESPECPROTO.enum_types_by_name['TypeSpecClass']
StructuredValue = _reflection.GeneratedProtocolMessageType('StructuredValue', (_message.Message,), {
  'DESCRIPTOR' : _STRUCTUREDVALUE,
  '__module__' : 'tensorboard.compat.proto.struct_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.StructuredValue)
  })
_sym_db.RegisterMessage(StructuredValue)

NoneValue = _reflection.GeneratedProtocolMessageType('NoneValue', (_message.Message,), {
  'DESCRIPTOR' : _NONEVALUE,
  '__module__' : 'tensorboard.compat.proto.struct_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.NoneValue)
  })
_sym_db.RegisterMessage(NoneValue)

ListValue = _reflection.GeneratedProtocolMessageType('ListValue', (_message.Message,), {
  'DESCRIPTOR' : _LISTVALUE,
  '__module__' : 'tensorboard.compat.proto.struct_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.ListValue)
  })
_sym_db.RegisterMessage(ListValue)

TupleValue = _reflection.GeneratedProtocolMessageType('TupleValue', (_message.Message,), {
  'DESCRIPTOR' : _TUPLEVALUE,
  '__module__' : 'tensorboard.compat.proto.struct_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.TupleValue)
  })
_sym_db.RegisterMessage(TupleValue)

DictValue = _reflection.GeneratedProtocolMessageType('DictValue', (_message.Message,), {

  'FieldsEntry' : _reflection.GeneratedProtocolMessageType('FieldsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DICTVALUE_FIELDSENTRY,
    '__module__' : 'tensorboard.compat.proto.struct_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.DictValue.FieldsEntry)
    })
  ,
  'DESCRIPTOR' : _DICTVALUE,
  '__module__' : 'tensorboard.compat.proto.struct_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.DictValue)
  })
_sym_db.RegisterMessage(DictValue)
_sym_db.RegisterMessage(DictValue.FieldsEntry)

PairValue = _reflection.GeneratedProtocolMessageType('PairValue', (_message.Message,), {
  'DESCRIPTOR' : _PAIRVALUE,
  '__module__' : 'tensorboard.compat.proto.struct_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.PairValue)
  })
_sym_db.RegisterMessage(PairValue)

NamedTupleValue = _reflection.GeneratedProtocolMessageType('NamedTupleValue', (_message.Message,), {
  'DESCRIPTOR' : _NAMEDTUPLEVALUE,
  '__module__' : 'tensorboard.compat.proto.struct_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.NamedTupleValue)
  })
_sym_db.RegisterMessage(NamedTupleValue)

TensorSpecProto = _reflection.GeneratedProtocolMessageType('TensorSpecProto', (_message.Message,), {
  'DESCRIPTOR' : _TENSORSPECPROTO,
  '__module__' : 'tensorboard.compat.proto.struct_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.TensorSpecProto)
  })
_sym_db.RegisterMessage(TensorSpecProto)

BoundedTensorSpecProto = _reflection.GeneratedProtocolMessageType('BoundedTensorSpecProto', (_message.Message,), {
  'DESCRIPTOR' : _BOUNDEDTENSORSPECPROTO,
  '__module__' : 'tensorboard.compat.proto.struct_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.BoundedTensorSpecProto)
  })
_sym_db.RegisterMessage(BoundedTensorSpecProto)

TypeSpecProto = _reflection.GeneratedProtocolMessageType('TypeSpecProto', (_message.Message,), {
  'DESCRIPTOR' : _TYPESPECPROTO,
  '__module__' : 'tensorboard.compat.proto.struct_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.TypeSpecProto)
  })
_sym_db.RegisterMessage(TypeSpecProto)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto'
  _DICTVALUE_FIELDSENTRY._options = None
  _DICTVALUE_FIELDSENTRY._serialized_options = b'8\001'
  _STRUCTUREDVALUE._serialized_start=177
  _STRUCTUREDVALUE._serialized_end=843
  _NONEVALUE._serialized_start=845
  _NONEVALUE._serialized_end=856
  _LISTVALUE._serialized_start=858
  _LISTVALUE._serialized_end=915
  _TUPLEVALUE._serialized_start=917
  _TUPLEVALUE._serialized_end=975
  _DICTVALUE._serialized_start=978
  _DICTVALUE._serialized_end=1118
  _DICTVALUE_FIELDSENTRY._serialized_start=1043
  _DICTVALUE_FIELDSENTRY._serialized_end=1118
  _PAIRVALUE._serialized_start=1120
  _PAIRVALUE._serialized_end=1189
  _NAMEDTUPLEVALUE._serialized_start=1191
  _NAMEDTUPLEVALUE._serialized_end=1262
  _TENSORSPECPROTO._serialized_start=1264
  _TENSORSPECPROTO._serialized_end=1379
  _BOUNDEDTENSORSPECPROTO._serialized_start=1382
  _BOUNDEDTENSORSPECPROTO._serialized_end=1590
  _TYPESPECPROTO._serialized_start=1593
  _TYPESPECPROTO._serialized_end=2099
  _TYPESPECPROTO_TYPESPECCLASS._serialized_start=1787
  _TYPESPECPROTO_TYPESPECCLASS._serialized_end=2099
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/compat/proto/cluster.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&tensorboard/compat/proto/cluster.proto\x12\x0btensorboard\"s\n\x06JobDef\x12\x0c\n\x04name\x18\x01 \x01(\t\x12-\n\x05tasks\x18\x02 \x03(\x0b\x32\x1e.tensorboard.JobDef.TasksEntry\x1a,\n\nTasksEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\".\n\nClusterDef\x12 \n\x03job\x18\x01 \x03(\x0b\x32\x13.tensorboard.JobDefB\x87\x01\n\x1aorg.tensorflow.distruntimeB\rClusterProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01\x62\x06proto3')



_JOBDEF = DESCRIPTOR.message_types_by_name['JobDef']
_JOBDEF_TASKSENTRY = _JOBDEF.nested_types_by_name['TasksEntry']
_CLUSTERDEF = DESCRIPTOR.message_types_by_name['ClusterDef']
JobDef = _reflection.GeneratedProtocolMessageType('JobDef', (_message.Message,), {

  'TasksEntry' : _reflection.GeneratedProtocolMessageType('TasksEntry', (_message.Message,), {
    'DESCRIPTOR' : _JOBDEF_TASKSENTRY,
    '__module__' : 'tensorboard.compat.proto.cluster_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.JobDef.TasksEntry)
    })
  ,
  'DESCRIPTOR' : _JOBDEF,
  '__module__' : 'tensorboard.compat.proto.cluster_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.JobDef)
  })
_sym_db.RegisterMessage(JobDef)
_sym_db.RegisterMessage(JobDef.TasksEntry)

ClusterDef = _reflection.GeneratedProtocolMessageType('ClusterDef', (_message.Message,), {
  'DESCRIPTOR' : _CLUSTERDEF,
  '__module__' : 'tensorboard.compat.proto.cluster_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.ClusterDef)
  })
_sym_db.RegisterMessage(ClusterDef)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\032org.tensorflow.distruntimeB\rClusterProtosP\001ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\370\001\001'
  _JOBDEF_TASKSENTRY._options = None
  _JOBDEF_TASKSENTRY._serialized_options = b'8\001'
  _JOBDEF._serialized_start=55
  _JOBDEF._serialized_end=170
  _JOBDEF_TASKSENTRY._serialized_start=126
  _JOBDEF_TASKSENTRY._serialized_end=170
  _CLUSTERDEF._serialized_start=172
  _CLUSTERDEF._serialized_end=218
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/plugins/histogram/plugin_data.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/tensorboard/plugins/histogram/plugin_data.proto\x12\x0btensorboard\"&\n\x13HistogramPluginData\x12\x0f\n\x07version\x18\x01 \x01(\x05\x62\x06proto3')



_HISTOGRAMPLUGINDATA = DESCRIPTOR.message_types_by_name['HistogramPluginData']
HistogramPluginData = _reflection.GeneratedProtocolMessageType('HistogramPluginData', (_message.Message,), {
  'DESCRIPTOR' : _HISTOGRAMPLUGINDATA,
  '__module__' : 'tensorboard.plugins.histogram.plugin_data_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.HistogramPluginData)
  })
_sym_db.RegisterMessage(HistogramPluginData)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _HISTOGRAMPLUGINDATA._serialized_start=64
  _HISTOGRAMPLUGINDATA._serialized_end=102
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/plugins/audio/plugin_data.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+tensorboard/plugins/audio/plugin_data.proto\x12\x0btensorboard\"\x9a\x01\n\x0f\x41udioPluginData\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\x37\n\x08\x65ncoding\x18\x02 \x01(\x0e\x32%.tensorboard.AudioPluginData.Encoding\x12\x1b\n\x13\x63onverted_to_tensor\x18\x03 \x01(\x08\" \n\x08\x45ncoding\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x07\n\x03WAV\x10\x0b\x62\x06proto3')



_AUDIOPLUGINDATA = DESCRIPTOR.message_types_by_name['AudioPluginData']
_AUDIOPLUGINDATA_ENCODING = _AUDIOPLUGINDATA.enum_types_by_name['Encoding']
AudioPluginData = _reflection.GeneratedProtocolMessageType('AudioPluginData', (_message.Message,), {
  'DESCRIPTOR' : _AUDIOPLUGINDATA,
  '__module__' : 'tensorboard.plugins.audio.plugin_data_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.AudioPluginData)
  })
_sym_db.RegisterMessage(AudioPluginData)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _AUDIOPLUGINDATA._serialized_start=61
  _AUDIOPLUGINDATA._serialized_end=215
  _AUDIOPLUGINDATA_ENCODING._serialized_start=183
  _AUDIOPLUGINDATA_ENCODING._serialized_end=215
# @@protoc_insertion_point(module_scope)



# Copyright 2019 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
# Lint as: python3
"""Provides authentication support for TensorBoardUploader."""


import datetime
import errno
import json
import os
import requests
import sys
import time
import webbrowser

import google_auth_oauthlib.flow as auth_flows
import grpc
import google.auth
import google.auth.transport.requests
import google.oauth2.credentials

from tensorboard.uploader import util
from tensorboard.util import tb_logging


logger = tb_logging.get_logger()


# OAuth2 scopes used for OpenID Connect:
# https://developers.google.com/identity/protocols/OpenIDConnect#scope-param
OPENID_CONNECT_SCOPES = (
    "openid",
    "https://www.googleapis.com/auth/userinfo.email",
)

# This config was downloaded from our GCP project at:
# console.cloud.google.com/apis/credentials?project=hosted-tensorboard-prod
# and in b/143316611.
#
# The client "secret" is considered public, as it's distributed to the devices
# where this runs. See:
# https://developers.google.com/identity/protocols/OAuth2?csw=1#installed
#
# See below for the config for another accepted credential.
_INSTALLED_APP_OAUTH_CLIENT_CONFIG = """
    {
        "installed":{
            "client_id":"373649185512-8v619h5kft38l4456nm2dj4ubeqsrvh6.apps.googleusercontent.com",
            "project_id":"hosted-tensorboard-prod",
            "auth_uri":"https://accounts.google.com/o/oauth2/auth",
            "token_uri":"https://oauth2.googleapis.com/token",
            "auth_provider_x509_cert_url":"https://www.googleapis.com/oauth2/v1/certs",
            "client_secret":"pOyAuU2yq2arsM98Bw5hwYtr",
            "redirect_uris":["http://localhost"]
        }
    }
"""


# These values can be updated with values from the well-known "discovery url":
# https://accounts.google.com/.well-known/openid-configuration
#
# See:
# developers.google.com/identity/openid-connect/openid-connect#discovery
_DEVICE_AUTH_CODE_URI = "https://oauth2.googleapis.com/device/code"


_LIMITED_INPUT_DEVICE_AUTH_GRANT_TYPE = (
    "urn:ietf:params:oauth:grant-type:device_code"
)

# This config was downloaded from our GCP project at:
# console.cloud.google.com/apis/credentials?project=hosted-tensorboard-prod
# and in b/262276562.
#
# Note that some of these fields are not really useful for this flow.
# It seems the limited-input device flow is not quite as well supported yet by
# neither the Google python oauth libraries, nor the GCP console, so this config
# does not match what we would need to authenticate this way (starting from the
# fact that it seems to be a config for an "installed" app), but we do use some
# of them (e.g. client_id and client_secret), along with other values defined in
# separate constants for this auth flow.
#
# The client "secret" is considered public, as it's distributed to the devices
# where this runs. See:
# https://developers.google.com/identity/protocols/oauth2/limited-input-device
#
# See above for the config for another accepted credential.
_LIMITED_INPUT_DEVICE_OAUTH_CLIENT_CONFIG = """
    {
        "installed":{
            "client_id":"373649185512-26ojik4u7dt0rdtfdmfnhpajqqh579qd.apps.googleusercontent.com",
            "project_id":"hosted-tensorboard-prod",
            "auth_uri":"https://accounts.google.com/o/oauth2/auth",
            "token_uri":"https://oauth2.googleapis.com/token",
            "auth_provider_x509_cert_url":"https://www.googleapis.com/oauth2/v1/certs",
            "client_secret":"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"
        }
    }
"""


# Components of the relative path (within the user settings directory) at which
# to store TensorBoard uploader credentials.
TENSORBOARD_CREDENTIALS_FILEPATH_PARTS = [
    "tensorboard",
    "credentials",
    "uploader-creds.json",
]


class CredentialsStore:
    """Private file store for a `google.oauth2.credentials.Credentials`."""

    _DEFAULT_CONFIG_DIRECTORY = object()  # Sentinel value.

    def __init__(self, user_config_directory=_DEFAULT_CONFIG_DIRECTORY):
        """Creates a CredentialsStore.

        Args:
          user_config_directory: Optional absolute path to the root directory for
            storing user configs, under which to store the credentials file. If not
            set, defaults to a platform-specific location. If set to None, the
            store is disabled (reads return None; write and clear are no-ops).
        """
        if user_config_directory is CredentialsStore._DEFAULT_CONFIG_DIRECTORY:
            user_config_directory = util.get_user_config_directory()
            if user_config_directory is None:
                logger.warning(
                    "Credentials caching disabled - no private config directory found"
                )
        if user_config_directory is None:
            self._credentials_filepath = None
        else:
            self._credentials_filepath = os.path.join(
                user_config_directory, *TENSORBOARD_CREDENTIALS_FILEPATH_PARTS
            )

    def read_credentials(self):
        """Returns the current `google.oauth2.credentials.Credentials`, or
        None."""
        if self._credentials_filepath is None:
            return None
        if os.path.exists(self._credentials_filepath):
            return (
                google.oauth2.credentials.Credentials.from_authorized_user_file(
                    self._credentials_filepath
                )
            )
        return None

    def write_credentials(self, credentials):
        """Writes a `google.oauth2.credentials.Credentials` to the store."""
        if not isinstance(credentials, google.oauth2.credentials.Credentials):
            raise TypeError(
                "Cannot write credentials of type %s" % type(credentials)
            )
        if self._credentials_filepath is None:
            return
        # Make the credential file private if not on Windows; on Windows we rely on
        # the default user config settings directory being private since we don't
        # have a straightforward way to make an individual file private.
        private = os.name != "nt"
        util.make_file_with_directories(
            self._credentials_filepath, private=private
        )
        data = {
            "refresh_token": credentials.refresh_token,
            "token_uri": credentials.token_uri,
            "client_id": credentials.client_id,
            "client_secret": credentials.client_secret,
            "scopes": credentials.scopes,
            "type": "authorized_user",
        }
        with open(self._credentials_filepath, "w") as f:
            json.dump(data, f)

    def clear(self):
        """Clears the store of any persisted credentials information."""
        if self._credentials_filepath is None:
            return
        try:
            os.remove(self._credentials_filepath)
        except OSError as e:
            if e.errno != errno.ENOENT:
                raise


def authenticate_user(
    force_console=False,
) -> google.oauth2.credentials.Credentials:
    """Makes the user authenticate to retrieve auth credentials.

    The default behavior is to use the [installed app flow](
    http://developers.google.com/identity/protocols/oauth2/native-app), in which
    a browser is started for the user to authenticate, along with a local web
    server. The authentication in the browser would produce a redirect response
    to `localhost` with an authorization code that would then be received by the
    local web server started here.

    The two most notable cases where the default flow is not well supported are:
    - When the uploader is run from a colab notebook.
    - Then the uploader is run via a remote terminal (SSH).

    If any of the following is true, a different auth flow will be used:
    - the flag `--auth_force_console` is set to true, or
    - a browser is not available, or
    - a local web server cannot be started

    In this case, a [limited-input device flow](
    http://developers.google.com/identity/protocols/oauth2/limited-input-device)
    will be used, in which the user is presented with a URL and a short code
    that they'd need to use to authenticate and authorize access in a separate
    browser or device. The uploader will poll for access until the access is
    granted or rejected, or the initiated authorization request expires.
    """
    scopes = OPENID_CONNECT_SCOPES
    # TODO(b/141721828): make auto-detection smarter, especially for macOS.
    if not force_console and os.getenv("DISPLAY"):
        try:
            client_config = json.loads(_INSTALLED_APP_OAUTH_CLIENT_CONFIG)
            flow = auth_flows.InstalledAppFlow.from_client_config(
                client_config, scopes=scopes
            )
            return flow.run_local_server(port=0)
        except webbrowser.Error:
            sys.stderr.write("Falling back to remote authentication flow...\n")

    client_config = json.loads(_LIMITED_INPUT_DEVICE_OAUTH_CLIENT_CONFIG)
    flow = _LimitedInputDeviceAuthFlow(client_config, scopes=scopes)
    return flow.run()


class _LimitedInputDeviceAuthFlow:
    """OAuth flow to authenticate using the limited-input device flow.

    See:
    http://developers.google.com/identity/protocols/oauth2/limited-input-device
    """

    def __init__(self, client_config, scopes):
        # The config is loaded from a json string from the GCP project, which
        # comes with an "installed" wrapper layer on the values we need.
        self._client_config = client_config["installed"]
        self._scopes = scopes

    def run(self) -> google.oauth2.credentials.Credentials:
        device_response = self._send_device_auth_request()
        prompt_message = (
            "To sign in with the TensorBoard uploader:\n"
            "\n"
            "1. On your computer or phone, visit:\n"
            "\n"
            "   {url}\n"
            "\n"
            "2. Sign in with your Google account, then enter:\n"
            "\n"
            "   {code}\n".format(
                url=device_response["verification_url"],
                code=device_response["user_code"],
            )
        )
        print(prompt_message)

        auth_response = self._poll_for_auth_token(
            device_code=device_response["device_code"],
            polling_interval=device_response["interval"],
            expiration_seconds=device_response["expires_in"],
        )

        return self._build_credentials(auth_response)

    def _send_device_auth_request(self):
        params = {
            "client_id": self._client_config["client_id"],
            "scope": " ".join(self._scopes),
        }
        r = requests.post(_DEVICE_AUTH_CODE_URI, data=params).json()
        if "device_code" not in r:
            raise RuntimeError(
                "There was an error while contacting Google's authorization "
                "server. Please try again later."
            )
        return r

    def _poll_for_auth_token(
        self, device_code: str, polling_interval: int, expiration_seconds: int
    ):
        token_uri = self._client_config["token_uri"]
        params = {
            "client_id": self._client_config["client_id"],
            "client_secret": self._client_config["client_secret"],
            "device_code": device_code,
            "grant_type": _LIMITED_INPUT_DEVICE_AUTH_GRANT_TYPE,
        }
        expiration_time = time.time() + expiration_seconds
        # Error cases documented in
        # https://developers.google.com/identity/protocols/oauth2/limited-input-device#step-6:-handle-responses-to-polling-requests
        while time.time() < expiration_time:
            resp = requests.post(token_uri, data=params)
            r = resp.json()
            if "access_token" in r:
                return r
            elif "error" in r and r["error"] == "authorization_pending":
                # Not really an error. This is the expected response when the
                # user has not yet granted access to this app.
                time.sleep(polling_interval)
            elif "error" in r and r["error"] == "slow_down":
                # We should be polling at the specified interval from the
                # previous response, so this error would be unexpected.
                # However, it is just a temporary/retryable error, so we can
                # poll a bit more slowly.
                polling_interval = int(polling_interval * 1.5)
                time.sleep(polling_interval)
            elif "error" in r and r["error"] == "access_denied":
                raise PermissionError("Access was denied by user.")
            elif resp.status_code in {400, 401}:
                raise ValueError("There must be an error in the request.")
            else:
                raise RuntimeError(
                    "An unexpected error occurred while waiting for "
                    "authorization."
                )
        raise TimeoutError("Timed out waiting for authorization.")

    def _build_credentials(
        self, auth_response
    ) -> google.oauth2.credentials.Credentials:

        expiration_datetime = datetime.datetime.utcfromtimestamp(
            int(time.time()) + auth_response["expires_in"]
        )
        return google.oauth2.credentials.Credentials(
            auth_response["access_token"],
            refresh_token=auth_response["refresh_token"],
            id_token=auth_response["id_token"],
            token_uri=self._client_config["token_uri"],
            client_id=self._client_config["client_id"],
            client_secret=self._client_config["client_secret"],
            scopes=self._scopes,
            expiry=expiration_datetime,
        )


class IdTokenAuthMetadataPlugin(grpc.AuthMetadataPlugin):
    """A `gRPC AuthMetadataPlugin` that uses ID tokens.

    This works like the existing `google.auth.transport.grpc.AuthMetadataPlugin`
    except that instead of always using access tokens, it preferentially uses the
    `Credentials.id_token` property if available (and logs an error otherwise).

    See http://www.grpc.io/grpc/python/grpc.html#grpc.AuthMetadataPlugin
    """

    def __init__(self, credentials, request):
        """Constructs an IdTokenAuthMetadataPlugin.

        Args:
          credentials (google.auth.credentials.Credentials): The credentials to
            add to requests.
          request (google.auth.transport.Request): A HTTP transport request object
            used to refresh credentials as needed.
        """
        super().__init__()
        if not isinstance(credentials, google.oauth2.credentials.Credentials):
            raise TypeError(
                "Cannot get ID tokens from credentials type %s"
                % type(credentials)
            )
        self._credentials = credentials
        self._request = request

    def __call__(self, context, callback):
        """Passes authorization metadata into the given callback.

        Args:
          context (grpc.AuthMetadataContext): The RPC context.
          callback (grpc.AuthMetadataPluginCallback): The callback that will
            be invoked to pass in the authorization metadata.
        """
        headers = {}
        self._credentials.before_request(
            self._request, context.method_name, context.service_url, headers
        )
        id_token = getattr(self._credentials, "id_token", None)
        if id_token:
            self._credentials.apply(headers, token=id_token)
        else:
            logger.error("Failed to find ID token credentials")
        # Pass headers as key-value pairs to match CallCredentials metadata.
        callback(list(headers.items()), None)


def id_token_call_credentials(credentials):
    """Constructs `grpc.CallCredentials` using
    `google.auth.Credentials.id_token`.

    Args:
      credentials (google.auth.credentials.Credentials): The credentials to use.

    Returns:
      grpc.CallCredentials: The call credentials.
    """
    request = google.auth.transport.requests.Request()
    return grpc.metadata_call_credentials(
        IdTokenAuthMetadataPlugin(credentials, request)
    )
